@@ -371,6 +371,18 @@ class _Slice(CubePartition):
         """
         return self._assembler.columns_margin
 
+    @lazyproperty
+    def columns_margin_proportion(self):
+        """1D or 2D np.float64 ndarray of weighted-proportion for each column of slice.
+
+        This array is 2D (a distinct margin value for each cell) when the rows dimension
+        is MR, because each MR-subvariable has its own weighted N. This is because not
+        every possible response is necessarily offered to every respondent.
+
+        In all other cases, the array is 1D, containing one value for each column.
+        """
+        return self._assembler.columns_margin_proportion
+
     @lazyproperty
     def columns_scale_mean(self):
         """Optional 1D np.float64 ndarray of scale mean for each column.
@@ -499,7 +511,9 @@ class _Slice(CubePartition):
 
         not_a_nan_index = ~np.isnan(self._rows_dimension_numeric_values)
         numeric_values = self._rows_dimension_numeric_values[not_a_nan_index]
-        counts = np.nan_to_num(self.counts[not_a_nan_index, :]).astype("int64")
+        counts = np.nan_to_num(
+            self._assembler.column_comparable_counts[not_a_nan_index, :]
+        ).astype("int64")
         scale_median = np.array(
             [
                 self._median(np.repeat(numeric_values, counts[:, i]))
@@ -614,6 +628,15 @@ class _Slice(CubePartition):
         column in the same row with a confidence interval meeting the threshold defined
         for this analysis.
         """
+        # If overlaps are defined, calculate significance based on them
+        if (
+            self.dimension_types[-1] == DT.MR
+            and self._cube.overlaps is not None
+            and self._cube.valid_overlaps is not None
+        ):
+            return self._assembler.pairwise_indices(self._alpha, self._only_larger)
+
+        # If overlaps are not defined, default to the legacy way of calculating sig
         return PairwiseSignificance.pairwise_indices(
             self, self._alpha, self._only_larger
         )
@@ -624,12 +647,21 @@ class _Slice(CubePartition):
 
         This value is None if no alternate threshold has been defined.
         """
-        return (
-            None
-            if self._alpha_alt is None
-            else PairwiseSignificance.pairwise_indices(
-                self, self._alpha_alt, self._only_larger
-            )
+
+        if self._alpha_alt is None:
+            return None
+
+        # If overlaps are defined, calculate significance based on them
+        if (
+            self.dimension_types[-1] == DT.MR
+            and self._cube.overlaps is not None
+            and self._cube.valid_overlaps is not None
+        ):
+            return self._assembler.pairwise_indices(self._alpha_alt, self._only_larger)
+
+        # If no overlaps are defined, default to legacy way of calculating pw indices
+        return PairwiseSignificance.pairwise_indices(
+            self, self._alpha_alt, self._only_larger
         )
 
     @lazyproperty
@@ -757,9 +789,26 @@ class _Slice(CubePartition):
 
     @lazyproperty
     def population_counts(self):
-        return (
-            self.table_proportions * self._population * self._cube.population_fraction
+        """2D np.float64 ndarray of population counts per cell.
+
+        The (estimated) population count is computed based on the `population` value
+        provided when the Slice is created. It is also adjusted to account for any
+        filters that were applied as part of the query.
+
+        If any of the dimensions (rows or columns) is a categorical date, the
+        appropriate percentages are used, to calculate the population counts, so as to
+        be the total amount among categorical dates.
+
+        Otherwise, table percents are used to calculate population counts.
+        """
+        proportions = (
+            self.row_proportions
+            if self.rows_dimension_type in (DT.CAT_DATE, DT.CA_SUBVAR)
+            else self.column_proportions
+            if self.columns_dimension_type in (DT.CAT_DATE, DT.CA_SUBVAR)
+            else self.table_proportions
         )
+        return proportions * self._population * self._cube.population_fraction
 
     @lazyproperty
     def population_counts_moe(self):
@@ -768,11 +817,24 @@ class _Slice(CubePartition):
         The values are represented as population estimates, analogue to the
         `population_counts` property. This means that the values will be presented by
         actual estimated counts of the population. The values can be np.nan when the
-        corresponding percentage is also np.nan, which happens when the respective
-        table margin is 0.
+        corresponding percentage is also np.nan, which happens when the
+        respective margin is 0.
+
+        When calculating the estimates of categorical dates, the total populatioin is
+        not "divided" between its categories, but rather considered constant for all
+        categorical dates (or waves). Hence, the different standard errors will be
+        applied in these specific cases (like the `row_std_err` or `column_std_err`).
+        If categorical dates are not involved, the standard `table_std_err` is used.
         """
+        std_err = (
+            self.row_std_err
+            if self.rows_dimension_type == DT.CAT_DATE
+            else self.column_std_err
+            if self.columns_dimension_type == DT.CAT_DATE
+            else self.table_std_err
+        )
         total_filtered_population = self._population * self._cube.population_fraction
-        return Z_975 * total_filtered_population * self.table_std_err
+        return Z_975 * total_filtered_population * std_err
 
     @lazyproperty
     def pvals(self):
@@ -923,6 +985,18 @@ class _Slice(CubePartition):
         """
         return self._assembler.rows_margin
 
+    @lazyproperty
+    def rows_margin_proportion(self):
+        """1D or 2D np.float64 ndarray of weighted-proportion for each column of slice.
+
+        This array is 2D (a distinct margin value for each cell) when the columns
+        dimension is MR, because each MR-subvariable has its own weighted N. This is
+        because not every possible response is necessarily offered to every respondent.
+
+        In all other cases, the array is 1D, containing one value for each column.
+        """
+        return self._assembler.rows_margin_proportion
+
     @lazyproperty
     def rows_scale_mean(self):
         """Optional 1D np.float64 ndarray of scale mean for each row.
@@ -1018,7 +1092,9 @@ class _Slice(CubePartition):
 
         not_a_nan_index = ~np.isnan(self._columns_dimension_numeric_values)
         numeric_values = self._columns_dimension_numeric_values[not_a_nan_index]
-        counts = np.nan_to_num(self.counts[:, not_a_nan_index]).astype("int64")
+        counts = np.nan_to_num(
+            self._assembler.row_comparable_counts[:, not_a_nan_index]
+        ).astype("int64")
         scale_median = np.array(
             [
                 self._median(np.repeat(numeric_values, counts[i, :]))
@@ -1468,6 +1544,14 @@ class _Strand(CubePartition):
         provided when the Strand is created. It is also adjusted to account for any
         filters that were applied as part of the query.
         """
+        # ---If the only dimension is a categorical date, we don't need to break up
+        # ---the population counts between its elements - it's constant throughout time
+        if self.rows_dimension_type == DT.CAT_DATE:
+            return np.full(
+                self.table_proportions.shape,
+                self._population * self._cube.population_fraction,
+            )
+
         return (
             self.table_proportions * self._population * self._cube.population_fraction
         )
