@@ -26,7 +26,6 @@ from cr.cube.matrix.subtotals import (
     NanSubtotals,
     SumSubtotals,
     TableStdErrSubtotals,
-    ZscoreSubtotals,
 )
 from cr.cube.util import lazyproperty
 
@@ -58,6 +57,17 @@ class Assembler(object):
         self._dimensions = dimensions
         self._slice_idx = slice_idx
 
+    @lazyproperty
+    def column_comparable_counts(self):
+        """2D np.float64 ndarray of counts comparable in the column dimension.
+
+        Comparable in the "column" dimension means that it's not a column subtotal
+        difference because these are not considered comparable since their bases
+        aren't the same. The values along these subtotal differences are defined as
+        np.nan.
+        """
+        return self._assemble_matrix(self._measures.column_comparable_counts.blocks)
+
     @lazyproperty
     def column_index(self):
         """2D np.float64 ndarray of column-index "percentage" for each table cell."""
@@ -108,19 +118,15 @@ class Assembler(object):
     def columns_base(self):
         """1D/2D np.float64 ndarray of unweighted-N for each slice column/cell."""
         # --- an MR_X slice produces a 2D column-base (each cell has its own N) ---
-        columns_base = self._cube_result_matrix.columns_base
         rows_dim_type = self._rows_dimension.dimension_type
+        columns_base = self._measures.columns_base
         if rows_dim_type in (DT.MR_SUBVAR, DT.NUM_ARRAY):
             return self._assemble_matrix(
                 SumSubtotals.blocks(columns_base, self._dimensions, diff_cols_nan=True)
             )
-
         # --- otherwise columns-base is a vector ---
         return self._assemble_vector(
-            columns_base,
-            self._column_subtotals,
-            self._column_order,
-            diffs_nan=True,
+            columns_base, self._column_subtotals, self._column_order, diffs_nan=True
         )
 
     @lazyproperty
@@ -147,6 +153,7 @@ class Assembler(object):
                 SumSubtotals.blocks(
                     self._cube_result_matrix.columns_margin,
                     self._dimensions,
+                    diff_cols_nan=True,
                 )
             )
 
@@ -155,6 +162,27 @@ class Assembler(object):
             self._cube_result_matrix.columns_margin,
             self._column_subtotals,
             self._column_order,
+            diffs_nan=True,
+        )
+
+    @lazyproperty
+    def columns_margin_proportion(self):
+        """1D/2D np.float64 ndarray of weighted-prop for each column of this slice."""
+        # --- an MR_X slice produces a 2D columns-margin (each cell has its own N) ---
+        denominator = self._cube_result_matrix.table_margin
+        if self._rows_dimension.dimension_type == DT.MR_SUBVAR:
+            return self._assemble_matrix(
+                SumSubtotals.blocks(
+                    self._cube_result_matrix.columns_margin / denominator,
+                    self._dimensions,
+                )
+            )
+
+        # --- otherwise columns-base is a vector ---
+        return self._assemble_vector(
+            self._cube_result_matrix.columns_margin / denominator,
+            self._column_subtotals,
+            self._column_order,
         )
 
     @lazyproperty
@@ -177,14 +205,35 @@ class Assembler(object):
         """
         return self._assemble_matrix(self._measures.means.blocks)
 
+    def pairwise_indices(self, alpha, only_larger):
+        """2D optional ndarray of tuple of int column-idxs pairwise-t threshold.
+
+        Raises `ValueError if the cube-result does not include `overlaps` cube-measures.
+        """
+        return self._assemble_matrix(
+            self._measures.pairwise_indices(alpha, only_larger).blocks
+        )
+
     def pairwise_means_indices(self, alpha, only_larger):
         """2D optional ndarray of tuple of int column-idxs means pairwise-t threshold.
 
         Raises `ValueError if the cube-result does not include `means` cube-measures.
         """
-        return self._assemble_matrix(
+        assembled_matrix = self._assemble_matrix(
             self._measures.pairwise_means_indices(alpha, only_larger).blocks
         )
+        ord_map = {k: v for v, k in enumerate(self._column_order)}
+
+        return np.array(
+            [
+                [
+                    tuple(ord_map[idx] for idx in idxs) if idxs is not None else None
+                    for idxs in row
+                ]
+                for row in assembled_matrix
+            ],
+            dtype=object,
+        )
 
     def pairwise_significance_p_vals(self, subvar_idx):
         """2D optional np.float64 ndarray of overlaps-p_vals matrices for subvar idx.
@@ -211,8 +260,9 @@ class Assembler(object):
 
         Raises `ValueError if the cube-result does not include `mean` cube-measures.
         """
+        base_column_idx = self._column_order[column_idx]
         return self._assemble_matrix(
-            self._measures.pairwise_significance_means_p_vals(column_idx).blocks
+            self._measures.pairwise_significance_means_p_vals(base_column_idx).blocks
         )
 
     def pairwise_significance_means_t_stats(self, column_idx):
@@ -220,8 +270,9 @@ class Assembler(object):
 
         Raises `ValueError if the cube-result does not include `mean` cube-measures.
         """
+        base_column_idx = self._column_order[column_idx]
         return self._assemble_matrix(
-            self._measures.pairwise_significance_means_t_stats(column_idx).blocks
+            self._measures.pairwise_significance_means_t_stats(base_column_idx).blocks
         )
 
     @lazyproperty
@@ -229,6 +280,17 @@ class Assembler(object):
         """2D np.float64/np.nan ndarray of p-value for each matrix cell."""
         return 2 * (1 - norm.cdf(np.abs(self.zscores)))
 
+    @lazyproperty
+    def row_comparable_counts(self):
+        """2D np.float64 ndarray of counts comparable in the row dimension.
+
+        Comparable in the "row" dimension means that it's not a row subtotal
+        difference because these are not considered comparable since their bases
+        aren't the same. The values along these subtotal differences are defined as
+        np.nan.
+        """
+        return self._assemble_matrix(self._measures.row_comparable_counts.blocks)
+
     @lazyproperty
     def row_labels(self):
         """1D str ndarray of row name for each matrix row.
@@ -322,13 +384,38 @@ class Assembler(object):
         if self._columns_dimension.dimension_type == DT.MR_SUBVAR:
             return self._assemble_matrix(
                 SumSubtotals.blocks(
-                    self._cube_result_matrix.rows_margin, self._dimensions
+                    self._cube_result_matrix.rows_margin,
+                    self._dimensions,
+                    diff_rows_nan=True,
                 )
             )
 
         # --- otherwise rows-margin is a vector ---
         return self._assemble_vector(
-            self._cube_result_matrix.rows_margin, self._row_subtotals, self._row_order
+            self._cube_result_matrix.rows_margin,
+            self._row_subtotals,
+            self._row_order,
+            diffs_nan=True,
+        )
+
+    @lazyproperty
+    def rows_margin_proportion(self):
+        """1D/2D np.float64 ndarray of weighted-proportion for each slice row/cell."""
+        # --- an X_MR slice produces a 2D rows-margin (each cell has its own N) ---
+        denominator = self._cube_result_matrix.table_margin
+        if self._columns_dimension.dimension_type == DT.MR_SUBVAR:
+            return self._assemble_matrix(
+                SumSubtotals.blocks(
+                    self._cube_result_matrix.rows_margin / denominator,
+                    self._dimensions,
+                )
+            )
+
+        # --- otherwise rows-margin is a vector ---
+        return self._assemble_vector(
+            self._cube_result_matrix.rows_margin / denominator,
+            self._row_subtotals,
+            self._row_order,
         )
 
     @lazyproperty
@@ -494,7 +581,8 @@ class Assembler(object):
     def total_share_sum(self):
         """2D optional np.float64 ndarray of total share of sum for each cell.
 
-        Raises `ValueError` if the cube-result does not include a sum cube-measure.
+        Raises `ValueError` if the cube-result does not include a sum
+        cube-measure.
         """
         return self._assemble_matrix(self._measures.total_share_sum.blocks)
 
@@ -516,22 +604,7 @@ class Assembler(object):
         deviations above (positive) or below (negative) the population mean a cell's
         value is.
         """
-        # --- punt except on CAT_X_CAT (if any MR dimensions it gets np.nans) ---
-        if (
-            self._rows_dimension.dimension_type == DT.MR_SUBVAR
-            or self._columns_dimension.dimension_type == DT.MR_SUBVAR
-        ):
-            return self._assemble_matrix(
-                NanSubtotals.blocks(self._cube_result_matrix.zscores, self._dimensions)
-            )
-
-        return self._assemble_matrix(
-            ZscoreSubtotals.blocks(
-                self._cube_result_matrix.zscores,
-                self._dimensions,
-                self._cube_result_matrix,
-            )
-        )
+        return self._assemble_matrix(self._measures.zscores.blocks)
 
     def _assemble_matrix(self, blocks):
         """Return 2D ndarray matrix assembled from `blocks`.
@@ -668,12 +741,11 @@ class _BaseOrderHelper(object):
         insertion, hiding, pruning, and ordering transforms specified in the
         rows-dimension.
         """
-        collation_method = dimensions[0].collation_method
-        HelperCls = (
-            _SortRowsByColumnValueHelper
-            if collation_method == CM.OPPOSING_ELEMENT
-            else _RowOrderHelper
-        )
+        order_spec = dimensions[0].order_spec
+        HelperCls = {
+            CM.OPPOSING_ELEMENT: _SortRowsByBaseColumnHelper,
+            CM.OPPOSING_INSERTION: _SortRowsByInsertedColumnHelper,
+        }.get(order_spec.collation_method, _RowOrderHelper)
 
         return HelperCls(dimensions, second_order_measures)._display_order
 
@@ -722,6 +794,36 @@ class _BaseOrderHelper(object):
             if N == 0
         )
 
+    @lazyproperty
+    def _measure(self):
+        """Second-order measure object providing values for sort.
+
+        This property is not used by some subclasses.
+        """
+        propname_by_measure = {
+            M.COLUMN_BASE_UNWEIGHTED: "column_unweighted_bases",
+            M.COLUMN_BASE_WEIGHTED: "column_weighted_bases",
+            M.COLUMN_PERCENT: "column_proportions",
+            M.MEAN: "means",
+            M.ROW_BASE_UNWEIGHTED: "row_unweighted_bases",
+            M.ROW_BASE_WEIGHTED: "row_weighted_bases",
+            M.ROW_PERCENT: "row_proportions",
+            M.TABLE_BASE_UNWEIGHTED: "table_unweighted_bases",
+            M.TABLE_BASE_WEIGHTED: "table_weighted_bases",
+            M.UNWEIGHTED_COUNT: "unweighted_counts",
+            M.WEIGHTED_COUNT: "weighted_counts",
+            M.Z_SCORE: "zscores",
+        }
+        measure = self._order_spec.measure
+        measure_propname = propname_by_measure.get(measure)
+
+        if measure_propname is None:
+            raise NotImplementedError(
+                "sort-by-value for measure '%s' is not yet supported" % measure
+            )
+
+        return getattr(self._second_order_measures, measure_propname)
+
     @lazyproperty
     def _order(self):
         """tuple of signed int idx for each sorted vector of measure matrix.
@@ -734,6 +836,16 @@ class _BaseOrderHelper(object):
             "%s must implement `._order`" % type(self).__name__
         )
 
+    @lazyproperty
+    def _order_spec(self):
+        """_OrderSpec object for dimension being sorted.
+
+        Provides access to ordering details like measure and sort-direction.
+        """
+        raise NotImplementedError(  # pragma: no cover
+            "%s must implement `._order_spec`" % type(self).__name__
+        )
+
     @lazyproperty
     def _prune_subtotals(self):
         """True if subtotal vectors need to be pruned, False otherwise."""
@@ -760,13 +872,21 @@ class _ColumnOrderHelper(_BaseOrderHelper):
         """
         CollatorCls = (
             ExplicitOrderCollator
-            if self._columns_dimension.collation_method == CM.EXPLICIT_ORDER
+            if self._order_spec.collation_method == CM.EXPLICIT_ORDER
             else PayloadOrderCollator
         )
         return CollatorCls.display_order(
             self._columns_dimension, self._empty_column_idxs
         )
 
+    @lazyproperty
+    def _order_spec(self):
+        """_OrderSpec object for the columns dimension.
+
+        Provides access to ordering details like measure and sort-direction.
+        """
+        return self._columns_dimension.order_spec
+
     @lazyproperty
     def _prune_subtotals(self):
         """True if subtotal columns need to be pruned, False otherwise.
@@ -795,11 +915,19 @@ class _RowOrderHelper(_BaseOrderHelper):
         """
         CollatorCls = (
             ExplicitOrderCollator
-            if self._rows_dimension.collation_method == CM.EXPLICIT_ORDER
+            if self._order_spec.collation_method == CM.EXPLICIT_ORDER
             else PayloadOrderCollator
         )
         return CollatorCls.display_order(self._rows_dimension, self._empty_row_idxs)
 
+    @lazyproperty
+    def _order_spec(self):
+        """_OrderSpec object for this row dimension.
+
+        Provides access to ordering details like measure and sort-direction.
+        """
+        return self._rows_dimension.order_spec
+
     @lazyproperty
     def _prune_subtotals(self):
         """True if subtotal rows need to be pruned, False otherwise.
@@ -814,7 +942,7 @@ class _RowOrderHelper(_BaseOrderHelper):
         )
 
 
-class _SortRowsByColumnValueHelper(_RowOrderHelper):
+class _SortRowsByBaseColumnHelper(_RowOrderHelper):
     """Orders elements by the values of an opposing base (not a subtotal) vector.
 
     This would be like "order rows in descending order by value of 'Strongly Agree'
@@ -822,22 +950,12 @@ class _SortRowsByColumnValueHelper(_RowOrderHelper):
     a matrix dimension has an opposing dimension.
     """
 
-    @lazyproperty
-    def _order(self):
-        """tuple of int element-idx specifying ordering of dimension elements."""
-        return SortByValueCollator.display_order(
-            self._rows_dimension,
-            self._element_values,
-            self._subtotal_values,
-            self._empty_row_idxs,
-        )
-
     @lazyproperty
     def _column_idx(self):
         """int index of column whose values the sort is based on."""
-        row_element_ids = self._rows_dimension.element_ids
-        sort_column_id = self._order_dict["element_id"]
-        return row_element_ids.index(sort_column_id)
+        column_element_ids = self._columns_dimension.element_ids
+        sort_column_id = self._order_spec.element_id
+        return column_element_ids.index(sort_column_id)
 
     @lazyproperty
     def _element_values(self):
@@ -850,26 +968,14 @@ class _SortRowsByColumnValueHelper(_RowOrderHelper):
         return measure_base_values[:, self._column_idx]
 
     @lazyproperty
-    def _measure(self):
-        """Second-order measure object providing values for sort."""
-        propname_by_keyname = {
-            M.COL_PERCENT.value: "column_proportions",
-            # --- add others as sort-by-value for those measures comes online ---
-        }
-        measure_keyname = self._order_dict["measure"]
-        measure_propname = propname_by_keyname.get(measure_keyname)
-
-        if measure_propname is None:
-            raise NotImplementedError(
-                "sort-by-value for measure '%s' is not yet supported" % measure_keyname
-            )
-
-        return getattr(self._second_order_measures, measure_propname)
-
-    @lazyproperty
-    def _order_dict(self):
-        """dict specifying ordering details like measure and sort-direction."""
-        return self._rows_dimension.order_dict
+    def _order(self):
+        """tuple of int element-idx specifying ordering of dimension elements."""
+        return SortByValueCollator.display_order(
+            self._rows_dimension,
+            self._element_values,
+            self._subtotal_values,
+            self._empty_row_idxs,
+        )
 
     @lazyproperty
     def _subtotal_values(self):
@@ -880,3 +986,49 @@ class _SortRowsByColumnValueHelper(_RowOrderHelper):
         """
         measure_subtotal_rows = self._measure.blocks[1][0]
         return measure_subtotal_rows[:, self._column_idx]
+
+
+class _SortRowsByInsertedColumnHelper(_RowOrderHelper):
+    """Orders rows by the values in an inserted column.
+
+    This would be like "order rows in descending order by value of 'Top 3' subtotal
+    column. An opposing-insertion ordering is only available on a matrix because only
+    a matrix dimension has an opposing dimension.
+    """
+
+    @lazyproperty
+    def _element_values(self):
+        """Sequence of insertion "body" values that form the basis for sort order.
+
+        There is one value per base row and values appear in payload order. These are
+        only the "base" insertion values and do not include intersections.
+        """
+        insertion_base_values = self._measure.blocks[0][1]
+        return insertion_base_values[:, self._insertion_idx]
+
+    @lazyproperty
+    def _insertion_idx(self):
+        """int index of insertion whose values the sort is based on."""
+        return self._columns_dimension.insertion_ids.index(
+            self._order_spec.insertion_id
+        )
+
+    @lazyproperty
+    def _order(self):
+        """tuple of int element-idx specifying ordering of dimension elements."""
+        return SortByValueCollator.display_order(
+            self._rows_dimension,
+            self._element_values,
+            self._subtotal_values,
+            self._empty_row_idxs,
+        )
+
+    @lazyproperty
+    def _subtotal_values(self):
+        """Sequence of row-subtotal intersection values that contribute to sort basis.
+
+        There is one value per row subtotal and values appear in payload (dimension)
+        insertion order.
+        """
+        intersections = self._measure.blocks[1][1]
+        return intersections[:, self._insertion_idx]
