@@ -7,8 +7,14 @@ from __future__ import division
 import numpy as np
 from scipy.stats import t
 
+from cr.cube.enums import DIMENSION_TYPE as DT
 from cr.cube.matrix.cubemeasure import CubeMeasures
-from cr.cube.matrix.subtotals import SumSubtotals, NanSubtotals
+from cr.cube.matrix.subtotals import (
+    SumSubtotals,
+    NanSubtotals,
+    NoneSubtotals,
+    ZscoreSubtotals,
+)
 from cr.cube.util import lazyproperty
 
 
@@ -26,6 +32,11 @@ class SecondOrderMeasures(object):
         self._dimensions = dimensions
         self._slice_idx = slice_idx
 
+    @lazyproperty
+    def column_comparable_counts(self):
+        """_ColumnComparableCounts measure object for this cube-result."""
+        return _ColumnComparableCounts(self._dimensions, self, self._cube_measures)
+
     @lazyproperty
     def column_proportions(self):
         """_ColumnProportions measure object for this cube-result."""
@@ -46,6 +57,11 @@ class SecondOrderMeasures(object):
         """_ColumnWeightedBases measure object for this cube-result."""
         return _ColumnWeightedBases(self._dimensions, self, self._cube_measures)
 
+    @lazyproperty
+    def columns_base(self):
+        """1D np.float64 ndarray of unweighted-N for each matrix column."""
+        return self._cube_measures.unweighted_cube_counts.columns_base
+
     @lazyproperty
     def columns_pruning_base(self):
         """1D np.float64 ndarray of unweighted-N for each matrix column."""
@@ -56,8 +72,14 @@ class SecondOrderMeasures(object):
         """_Means measure object for this cube-result"""
         return _Means(self._dimensions, self, self._cube_measures)
 
+    def pairwise_indices(self, alpha, only_larger):
+        """_PairwiseIndices measure object for this cube-result"""
+        return _PairwiseIndices(
+            self._dimensions, self, self._cube_measures, alpha, only_larger
+        )
+
     def pairwise_means_indices(self, alpha, only_larger):
-        """_PairwiseMeansIndices measure object for this cube-result"""
+        """_PairwiseIndices measure object for this cube-result"""
         return _PairwiseMeansIndices(
             self._dimensions, self, self._cube_measures, alpha, only_larger
         )
@@ -91,6 +113,11 @@ class SecondOrderMeasures(object):
             self._dimensions, self, self._cube_measures, column_idx
         )
 
+    @lazyproperty
+    def row_comparable_counts(self):
+        """_RowComparableCounts measure object for this cube-result."""
+        return _RowComparableCounts(self._dimensions, self, self._cube_measures)
+
     @lazyproperty
     def row_proportions(self):
         """_RowProportions measure object for this cube-result."""
@@ -151,6 +178,11 @@ class SecondOrderMeasures(object):
         """_WeightedCounts measure object for this cube-result."""
         return _WeightedCounts(self._dimensions, self, self._cube_measures)
 
+    @lazyproperty
+    def zscores(self):
+        """_Zscores measure object for this cube-result."""
+        return _Zscores(self._dimensions, self, self._cube_measures)
+
     @lazyproperty
     def _cube_measures(self):
         """CubeMeasures collection object for this cube-result.
@@ -242,6 +274,32 @@ class _BaseSecondOrderMeasure(object):
         return self._cube_measures.weighted_cube_counts
 
 
+class _ColumnComparableCounts(_BaseSecondOrderMeasure):
+    """Provides the column-comparable count measure for a matrix.
+
+    Column-comparable Counts is a 2D np.float64 ndarray of the counts, with values in the
+    column of a subtotal difference overridden as np.nan because they do not share the
+    same base and so are "not comparable" when calculating other measures along the
+    column, like column proportions.
+    """
+
+    @lazyproperty
+    def blocks(self):
+        """Nested list of the four 2D ndarray "blocks" making up this measure.
+
+        These are the base-values, the column-subtotals, the row-subtotals, and the
+        subtotal intersection-cell values.
+
+        Column-comparable counts are the same as counts, exccept that they are undefined
+        for columns of subtotal differences.
+        """
+        return SumSubtotals.blocks(
+            self._weighted_cube_counts.weighted_counts,
+            self._dimensions,
+            diff_cols_nan=True,
+        )
+
+
 class _ColumnProportions(_BaseSecondOrderMeasure):
     """Provides the column-proportions measure for a matrix.
 
@@ -259,11 +317,7 @@ class _ColumnProportions(_BaseSecondOrderMeasure):
         Column-proportions are counts divided by the column base, except that they are
         undefined for columns with subtotal differences.
         """
-        count_blocks = SumSubtotals.blocks(
-            self._weighted_cube_counts.weighted_counts,
-            self._dimensions,
-            diff_cols_nan=True,
-        )
+        count_blocks = self._second_order_measures.column_comparable_counts.blocks
         weighted_base_blocks = self._second_order_measures.column_weighted_bases.blocks
 
         # --- do not propagate divide-by-zero warnings to stderr ---
@@ -478,6 +532,75 @@ class _Means(_BaseSecondOrderMeasure):
         )
 
 
+class _PairwiseIndices(_BaseSecondOrderMeasure):
+    """Provides pairwise significance indices measure for matrix."""
+
+    def __init__(
+        self, dimensions, second_order_measures, cube_measures, alpha, only_larger
+    ):
+        super(_PairwiseIndices, self).__init__(
+            dimensions, second_order_measures, cube_measures
+        )
+        self._alpha = alpha
+        self._only_larger = only_larger
+
+    @lazyproperty
+    def blocks(self):
+        """2D array of the four 2D "blocks" making up this measure."""
+        pairwise_indices = np.array(
+            [self._pairwise_indices(v.p_vals, v.t_stats) for v in self._values]
+        ).T
+        return NoneSubtotals.blocks(pairwise_indices, self._dimensions)
+
+    def _pairwise_indices(self, p_vals, t_stats):
+        """1D ndarray containing tuples of int pairwise indices of each column."""
+        significance = p_vals < self._alpha
+        if self._only_larger:
+            significance = np.logical_and(t_stats < 0, significance)
+        col_significance = np.empty((len(significance),), dtype=object)
+        col_significance[:] = [tuple(np.where(sig_row)[0]) for sig_row in significance]
+        return col_significance
+
+    @lazyproperty
+    def _values(self):
+        """list of _PairwiseSigPVals tests objects.
+
+        Result has as many elements as there are columns in the slice. Each
+        significance test contains `p_vals` and `t_stats` significance tests.
+        """
+        return [
+            _PairwiseSigPVals(
+                self._dimensions,
+                self._second_order_measures,
+                self._cube_measures,
+                col_idx,
+            )
+            for col_idx in range(
+                self._cube_measures.cube_overlaps.selected_bases.shape[1]
+            )
+        ]
+
+
+class _PairwiseMeansIndices(_PairwiseIndices):
+    """Provides pairwise means significance indices measure for matrix."""
+
+    @lazyproperty
+    def _values(self):
+        """list of _PairwiseMeansSigPVals tests objects.
+        Result has as many elements as there are columns in the slice. Each
+        significance test contains `p_vals` and `t_stats` significance tests.
+        """
+        return [
+            _PairwiseMeansSigPVals(
+                self._dimensions,
+                self._second_order_measures,
+                self._cube_measures,
+                col_idx,
+            )
+            for col_idx in range(self._cube_measures.cube_means.means.shape[1])
+        ]
+
+
 class _PairwiseSigTStats(_BaseSecondOrderMeasure):
     """Provides pairwise significance t-stats measure for matrix and selected subvar.
 
@@ -495,20 +618,20 @@ class _PairwiseSigTStats(_BaseSecondOrderMeasure):
     @lazyproperty
     def blocks(self):
         """2D array of the four 2D "blocks" making up this measure."""
-        return NanSubtotals.blocks(self._t_stats, self._dimensions)
+        return NanSubtotals.blocks(self.t_stats, self._dimensions)
 
     @lazyproperty
     def _n_rows(self):
         """int number of rows in the matrix."""
-        return self._cube_measures.cube_overlaps.overlaps.shape[0]
+        return self._cube_measures.cube_overlaps.valid_bases.shape[0]
 
     @lazyproperty
     def _n_subvars(self):
         """int number of columns (subvariables) in the matrix."""
-        return self._cube_measures.cube_overlaps.overlaps.shape[1]
+        return self._cube_measures.cube_overlaps.valid_bases.shape[1]
 
     @lazyproperty
-    def _t_stats(self):
+    def t_stats(self):
         """2D ndarray of float64 representing t-stats for pairwise MR testing.
 
         For each (category) row, we calculate the test statistic of the overlap between
@@ -521,8 +644,9 @@ class _PairwiseSigTStats(_BaseSecondOrderMeasure):
             [
                 [
                     _PairwiseSignificaneBetweenSubvariablesHelper(
-                        self._cube_measures.cube_overlaps.overlaps[row_idx],
-                        self._cube_measures.cube_overlaps.valid_overlaps[row_idx],
+                        self._second_order_measures.column_proportions.blocks[0][0],
+                        self._cube_measures.cube_overlaps,
+                        row_idx,
                         self._selected_subvar_idx,
                         subvar_idx,
                     ).t_stats
@@ -542,10 +666,10 @@ class _PairwiseSigPVals(_PairwiseSigTStats):
     @lazyproperty
     def blocks(self):
         """2D array of the four 2D "blocks" making up this measure."""
-        return NanSubtotals.blocks(self._p_vals, self._dimensions)
+        return NanSubtotals.blocks(self.p_vals, self._dimensions)
 
     @lazyproperty
-    def _p_vals(self):
+    def p_vals(self):
         """2D ndarray of float64 representing p-vals for pairwise MR testing.
 
         For each (category) row, we calculate the test significance of the overlap
@@ -558,8 +682,9 @@ class _PairwiseSigPVals(_PairwiseSigTStats):
             [
                 [
                     _PairwiseSignificaneBetweenSubvariablesHelper(
-                        self._cube_measures.cube_overlaps.overlaps[row_idx],
-                        self._cube_measures.cube_overlaps.valid_overlaps[row_idx],
+                        self._second_order_measures.column_proportions.blocks[0][0],
+                        self._cube_measures.cube_overlaps,
+                        row_idx,
                         self._selected_subvar_idx,
                         subvar_idx,
                     ).p_vals
@@ -572,7 +697,6 @@ class _PairwiseSigPVals(_PairwiseSigTStats):
 
 class _PairwiseMeansSigTStats(_BaseSecondOrderMeasure):
     """Provides pairwise means significance t-stats measure for matrix.
-
     Pairwise significance is calculated for each selected column separately.
     """
 
@@ -592,25 +716,26 @@ class _PairwiseMeansSigTStats(_BaseSecondOrderMeasure):
     @lazyproperty
     def t_stats(self):
         """2D ndarray of float64 representing t-stats for means pairwise testing.
-
         Calculate the level of significance for the difference of two means from the
         selected column different from a hypothesized value.
         t = (x̄1 - x̄2 - μ) √ (N / s2)
         s2 = var1 + var2 - 2 * covar12 - covar12 in this specific case is 0.
         """
+        if self._selected_column_idx < 0:
+            return np.full(self._cube_measures.cube_stddev.stddev.shape, np.nan)
+
         means = self._cube_measures.cube_means.means
         variance = np.power(self._cube_measures.cube_stddev.stddev, 2)
         col_bases = self._cube_measures.unweighted_cube_counts.column_bases
 
         combined_variance = variance[:, self._selected_column_idx] + variance.T
-        diff = means[:, self._selected_column_idx] - means.T
+        diff = means.T - means[:, self._selected_column_idx]
         n = col_bases[:, self._selected_column_idx] + col_bases.T
         return diff.T * np.sqrt(n.T / combined_variance.T)
 
 
 class _PairwiseMeansSigPVals(_PairwiseMeansSigTStats):
     """Provides pairwise means significance p-vals measure for matrix.
-
     Pairwise significance is calculated for each selected column separately.
     """
 
@@ -628,54 +753,30 @@ class _PairwiseMeansSigPVals(_PairwiseMeansSigTStats):
         return 2 * (1 - t.cdf(abs(self.t_stats), df=df.T))
 
 
-class _PairwiseMeansIndices(_BaseSecondOrderMeasure):
-    """Provides pairwise means significance indices measure for matrix."""
+class _RowComparableCounts(_BaseSecondOrderMeasure):
+    """Provides the row-comparable count measure for a matrix.
 
-    def __init__(
-        self, dimensions, second_order_measures, cube_measures, alpha, only_larger
-    ):
-        super(_PairwiseMeansIndices, self).__init__(
-            dimensions, second_order_measures, cube_measures
-        )
-        self._alpha = alpha
-        self._only_larger = only_larger
+    row-comparable Counts is a 2D np.float64 ndarray of the counts, with values in the
+    row of a subtotal difference overridden as np.nan because they do not share the
+    same base and so are "not comparable" when calculating other measures along the
+    row, like row proportions.
+    """
 
     @lazyproperty
     def blocks(self):
-        """2D array of the four 2D "blocks" making up this measure."""
-        pairwise_indices = np.array(
-            [self._pairwise_indices(v.p_vals, v.t_stats) for v in self._values]
-        ).T
-        blocks = np.array(
-            [[tuple(cell) for cell in row] for row in pairwise_indices], dtype=tuple
-        )
-        return NanSubtotals.blocks(blocks, self._dimensions)
+        """Nested list of the four 2D ndarray "blocks" making up this measure.
 
-    def _pairwise_indices(self, p_vals, t_stats):
-        """1D ndarray containing tuples of int pairwise indices of each column."""
-        significance = p_vals < self._alpha
-        if self._only_larger:
-            significance = np.logical_and(t_stats < 0, significance)
-        col_significance = np.empty((len(significance),), dtype=object)
-        col_significance[:] = [tuple(np.where(sig_row)[0]) for sig_row in significance]
-        return col_significance
+        These are the base-values, the column-subtotals, the row-subtotals, and the
+        subtotal intersection-cell values.
 
-    @lazyproperty
-    def _values(self):
-        """list of _PairwiseMeansSigPVals tests objects.
-
-        Result has as many elements as there are columns in the slice. Each
-        significance test contains `p_vals` and `t_stats` significance tests.
+        Row-comparable counts are the same as counts, exccept that they are undefined
+        for rows of subtotal differences.
         """
-        return [
-            _PairwiseMeansSigPVals(
-                self._dimensions,
-                self._second_order_measures,
-                self._cube_measures,
-                col_idx,
-            )
-            for col_idx in range(self._cube_measures.cube_means.means.shape[1])
-        ]
+        return SumSubtotals.blocks(
+            self._weighted_cube_counts.weighted_counts,
+            self._dimensions,
+            diff_rows_nan=True,
+        )
 
 
 class _RowProportions(_BaseSecondOrderMeasure):
@@ -692,14 +793,9 @@ class _RowProportions(_BaseSecondOrderMeasure):
         These are the base-values, the column-subtotals, the row-subtotals, and the
         subtotal intersection-cell values.
 
-        Row-proportions are counts divided by the row base, except that they are
-        undefined for rows with subtotal differences.
+        Row-proportions are counts divided by the row comparable counts.
         """
-        count_blocks = SumSubtotals.blocks(
-            self._weighted_cube_counts.weighted_counts,
-            self._dimensions,
-            diff_rows_nan=True,
-        )
+        count_blocks = self._second_order_measures.row_comparable_counts.blocks
         weighted_base_blocks = self._second_order_measures.row_weighted_bases.blocks
 
         # --- do not propagate divide-by-zero warnings to stderr ---
@@ -1193,8 +1289,12 @@ class _UnweightedCounts(_BaseSecondOrderMeasure):
         These are the base-values, the column-subtotals, the row-subtotals, and the
         subtotal intersection-cell values.
         """
+        diff_nans = self._unweighted_cube_counts.diff_nans
         return SumSubtotals.blocks(
-            self._unweighted_cube_counts.unweighted_counts, self._dimensions
+            self._unweighted_cube_counts.unweighted_counts,
+            self._dimensions,
+            diff_cols_nan=diff_nans,
+            diff_rows_nan=diff_nans,
         )
 
 
@@ -1204,20 +1304,38 @@ class _WeightedCounts(_BaseSecondOrderMeasure):
     @lazyproperty
     def blocks(self):
         """2D array of the four 2D "blocks" making up this measure."""
+        diff_nans = self._weighted_cube_counts.diff_nans
         return SumSubtotals.blocks(
-            self._weighted_cube_counts.weighted_counts, self._dimensions
+            self._weighted_cube_counts.weighted_counts,
+            self._dimensions,
+            diff_cols_nan=diff_nans,
+            diff_rows_nan=diff_nans,
         )
 
 
+class _Zscores(_BaseSecondOrderMeasure):
+    """Provides the zscore measure for a matrix."""
+
+    @lazyproperty
+    def blocks(self):
+        """2D array of the four 2D "blocks" making up this measure."""
+        base_values = self._weighted_cube_counts.zscores
+        dimension_types = tuple(d.dimension_type for d in self._dimensions)
+        if DT.MR_SUBVAR in dimension_types:
+            return NanSubtotals.blocks(base_values, self._dimensions)
+        return ZscoreSubtotals.blocks(self._weighted_cube_counts, self._dimensions)
+
+
 # === PAIRWISE HELPERS ===
 
 
 class _PairwiseSignificaneBetweenSubvariablesHelper(object):
     """Helper for calculating overlaps significance between subvariables."""
 
-    def __init__(self, overlaps, valid_overlaps, idx_a, idx_b):
-        self._overlaps = overlaps
-        self._valid_overlaps = valid_overlaps
+    def __init__(self, column_proportions, cube_overlaps, row_idx, idx_a, idx_b):
+        self._column_proportions = column_proportions
+        self._cube_overlaps = cube_overlaps
+        self._row_idx = row_idx
         self._idx_a = idx_a
         self._idx_b = idx_b
 
@@ -1239,8 +1357,12 @@ class _PairwiseSignificaneBetweenSubvariablesHelper(object):
         Sa, Sb, Sab = self._selected_counts
         Na, Nb, Nab = self._valid_counts
         pa, pb, pab = Sa / Na, Sb / Nb, Sab / Nab
+        col_prop_a = self._column_proportions[self._row_idx, self._idx_a]
+        col_prop_b = self._column_proportions[self._row_idx, self._idx_b]
 
-        return (pa - pb) / np.sqrt(
+        # ---Subtract the selected column from the "variable" column, to get
+        # ---the correct sign of the test statistic (hence b-a, and not a-b).
+        return (col_prop_b - col_prop_a) / np.sqrt(
             1 / self._df * (pa * (1 - pa) + pb * (1 - pb) + 2 * pa * pb - 2 * pab)
         )
 
@@ -1262,9 +1384,9 @@ class _PairwiseSignificaneBetweenSubvariablesHelper(object):
         be float64 when the overlaps result is weighted.
         """
         return (
-            self._overlaps[self._idx_a, self._idx_a],
-            self._overlaps[self._idx_b, self._idx_b],
-            self._overlaps[self._idx_a, self._idx_b],
+            self._cube_overlaps.selected_bases[self._row_idx, self._idx_a, self._idx_a],
+            self._cube_overlaps.selected_bases[self._row_idx, self._idx_b, self._idx_b],
+            self._cube_overlaps.selected_bases[self._row_idx, self._idx_a, self._idx_b],
         )
 
     @lazyproperty
@@ -1276,7 +1398,7 @@ class _PairwiseSignificaneBetweenSubvariablesHelper(object):
         They can only be float64 when the overlaps result is weighted.
         """
         return (
-            self._valid_overlaps[self._idx_a, self._idx_a],
-            self._valid_overlaps[self._idx_b, self._idx_b],
-            self._valid_overlaps[self._idx_a, self._idx_b],
+            self._cube_overlaps.valid_bases[self._row_idx, self._idx_a, self._idx_a],
+            self._cube_overlaps.valid_bases[self._row_idx, self._idx_b, self._idx_b],
+            self._cube_overlaps.valid_bases[self._row_idx, self._idx_a, self._idx_b],
         )
