@@ -136,17 +136,23 @@ class NanSubtotals(_BaseSubtotals):
     Each subtotal value (and intersection value) is `np.nan`.
     """
 
+    filler = np.nan
+
     def _intersection(self, row_subtotal, column_subtotal):
         """Unconditionally return np.nan for each intersection cell."""
-        return np.nan
+        return self.filler
 
     def _subtotal_column(self, subtotal):
         """Return (n_rows,) ndarray of np.nan values."""
-        return np.full(self._nrows, np.nan)
+        return np.full(self._nrows, self.filler)
 
     def _subtotal_row(self, subtotal):
         """Return (n_cols,) ndarray of np.nan values."""
-        return np.full(self._ncols, np.nan)
+        return np.full(self._ncols, self.filler)
+
+
+class NoneSubtotals(NanSubtotals):
+    filler = None
 
 
 class SumSubtotals(_BaseSubtotals):
@@ -383,29 +389,82 @@ class ZscoreSubtotals(_BaseSubtotals):
     subtotals to be computed (elsewhere) as `np.nan`.
     """
 
-    def __init__(self, base_values, dimensions, cube_result_matrix):
-        super(ZscoreSubtotals, self).__init__(base_values, dimensions)
-        self._cube_result_matrix = cube_result_matrix
+    def __init__(
+        self,
+        weighted_counts,
+        dimensions,
+        diff_cols_nan=True,
+        diff_rows_nan=True,
+    ):
+        super(ZscoreSubtotals, self).__init__(weighted_counts, dimensions)
+        self._weighted_counts = weighted_counts
+        self._diff_cols_nan = diff_cols_nan
+        self._diff_rows_nan = diff_rows_nan
 
     @classmethod
-    def blocks(cls, base_values, dimensions, cube_result_matrix):
+    def blocks(
+        cls, weighted_counts, dimensions, diff_cols_nan=True, diff_rows_nan=True
+    ):
         """Return base, row and col insertion, and intersection matrices.
 
         These are in the form ready for assembly.
+
+        Keyword arguments:
+        `diff_cols_nan` -- Overrides subtotal differences in the columns direction eg
+        for column bases (default False)
+        `diff_rows_nan` -- Overrides subtotal differences in the rows direction eg for
+        row bases (default False)
         """
-        return cls(base_values, dimensions, cube_result_matrix)._blocks
+        return cls(weighted_counts, dimensions, diff_cols_nan, diff_rows_nan)._blocks
+
+    @classmethod
+    def intersections(
+        cls, weighted_counts, dimensions, diff_cols_nan=True, diff_rows_nan=True
+    ):
+        """Return (n_row_subtotals, n_col_subtotals) ndarray of intersection values.
+
+        An intersection value arises where a row-subtotal crosses a column-subtotal.
+
+        Keyword arguments:
+        `diff_cols_nan` -- Overrides subtotal differences in the columns direction eg
+        for column bases (default False)
+        `diff_rows_nan` -- Overrides subtotal differences in the rows direction eg for
+        row bases (default False)
+        """
+        return cls(
+            weighted_counts, dimensions, diff_cols_nan, diff_rows_nan
+        )._intersections
 
     @lazyproperty
     def _base_counts(self):
-        """2D np.float64 ndarray of weighted-count for each cell of base matrix."""
-        return self._cube_result_matrix.weighted_counts
+        """2D float64 nparray of weighted counts."""
+        return self._weighted_counts.weighted_counts
+
+    @lazyproperty
+    def _blocks(self):
+        """base, row and col insertion, and intersection matrices."""
+        return [
+            [self._weighted_counts.zscores, self._subtotal_columns],
+            [self._subtotal_rows, self._intersections],
+        ]
+
+    @lazyproperty
+    def _columns_margin(self):
+        """2D np.float64 ndarray of weighted-N for each matrix column/cell."""
+        return self._weighted_counts.columns_margin
 
     def _intersection(self, row_subtotal, column_subtotal):
         """Return value for intersection of `row_subtotal` and `column_subtotal`."""
         # --- Cannot calculate if there are any subtrahends ---
+        col_has_subs = len(column_subtotal.subtrahend_idxs) > 0
+        row_has_subs = len(row_subtotal.subtrahend_idxs) > 0
+
         if (
-            len(row_subtotal.subtrahend_idxs) > 0
-            or len(column_subtotal.subtrahend_idxs) > 0
+            # --- Intersections of subtotal differences are undefined ---
+            (col_has_subs and row_has_subs)
+            # --- Also need to respect diff_cols_nan/diff_rows_nan
+            or (col_has_subs and self._diff_cols_nan)
+            or (row_has_subs and self._diff_rows_nan)
         ):
             return np.nan
 
@@ -420,9 +479,7 @@ class ZscoreSubtotals(_BaseSubtotals):
         subtotal_margin = np.sum(row_subtotal_counts)
 
         # --- opposite-margin is scalar because no MR dimensions ---
-        opposite_margin = np.sum(
-            self._cube_result_matrix.columns_margin[column_subtotal.addend_idxs]
-        )
+        opposite_margin = np.sum(self._columns_margin[column_subtotal.addend_idxs])
 
         # --- table_margin is scalar because no MR dimensions ---
         table_margin = self._table_margin
@@ -445,10 +502,25 @@ class ZscoreSubtotals(_BaseSubtotals):
         with np.errstate(divide="ignore", invalid="ignore"):
             return residuals / np.sqrt(variance)
 
+    @lazyproperty
+    def _ncols(self):
+        """int count of columns in base-matrix."""
+        return self._base_counts.shape[1]
+
+    @lazyproperty
+    def _nrows(self):
+        """int count of rows in base-matrix."""
+        return self._base_counts.shape[0]
+
+    @lazyproperty
+    def _rows_margin(self):
+        """2D np.float64 ndarray of weighted-N for each matrix row/cell."""
+        return self._weighted_counts.rows_margin
+
     def _subtotal_column(self, subtotal):
         """Return (n_rows,) ndarray of zscore `subtotal` value."""
         # --- Cannot calculate if there are any subtrahends ---
-        if len(subtotal.subtrahend_idxs) > 0:
+        if self._diff_cols_nan and len(subtotal.subtrahend_idxs) > 0:
             return np.full(self._nrows, np.nan)
 
         # --- the weighted-counts version of this subtotal-column ---
@@ -458,7 +530,7 @@ class ZscoreSubtotals(_BaseSubtotals):
         subtotal_margin = np.sum(subtotal_counts)
 
         # --- base-rows-margin is 1D because no MR dimensions ---
-        opposing_margin = self._cube_result_matrix.rows_margin
+        opposing_margin = self._rows_margin
 
         # --- table_margin is scalar because no MR dimensions ---
         table_margin = self._table_margin
@@ -484,7 +556,7 @@ class ZscoreSubtotals(_BaseSubtotals):
     def _subtotal_row(self, subtotal):
         """Return (n_cols,) ndarray of z-score `subtotal` value."""
         # --- Cannot calculate if there are any subtrahends ---
-        if len(subtotal.subtrahend_idxs) > 0:
+        if self._diff_rows_nan and len(subtotal.subtrahend_idxs) > 0:
             return np.full(self._ncols, np.nan)
 
         # --- the weighted-counts version of this subtotal-row ---
@@ -494,7 +566,7 @@ class ZscoreSubtotals(_BaseSubtotals):
         subtotal_margin = np.sum(subtotal_counts)
 
         # --- base-cols-margin is 1D because no MR dimensions ---
-        opposing_margin = self._cube_result_matrix.columns_margin
+        opposing_margin = self._columns_margin
 
         # --- table_margin is scalar because no MR dimensions ---
         table_margin = self._table_margin
@@ -519,5 +591,5 @@ class ZscoreSubtotals(_BaseSubtotals):
 
     @lazyproperty
     def _table_margin(self):
-        """Scalar or ndarray table-margin of cube-result matrix."""
-        return self._cube_result_matrix.table_margin
+        """Scalar np.float64 weighted-N for overall table."""
+        return self._weighted_counts.table_margin
