@@ -1513,26 +1513,17 @@ class DescribeCubemeasure(object):
         )
 
         assert overlaps_measure.__class__.__name__ == "_CatXMrOverlaps"
-        assert overlaps_measure.overlaps.tolist() == [
+        assert overlaps_measure.selected_bases.tolist() == [
             [
-                # A, B, C
-                [0, 0, 0],  # A
-                [0, 1, 1],  # B
-                [0, 1, 2],  # C
-            ],  # G[0] == 2
+                [1.0, 0.0, 0.0],
+                [0.0, 1.0, 1.0],
+                [0.0, 1.0, 2.0],
+            ],
             [
-                # A, B, C
-                [1, 0, 0],
-                [0, 0, 0],
-                [0, 0, 0],
-            ],  # G[1] == 1
-            # --- Missing categories are not shown by cube by default
-            # [
-            #     # A, B, C
-            #     [2, 1, 1],
-            #     [1, 1, 1],
-            #     [1, 1, 1],
-            # ],  # G[2] == missing
+                [1.0, 0.0, 0.0],
+                [0.0, 1.0, 1.0],
+                [0.0, 1.0, 2.0],
+            ],
         ]
 
     def it_provides_valid_overlaps_for_cat_x_mr_sub_x_mr_sel(self):
@@ -1543,16 +1534,16 @@ class DescribeCubemeasure(object):
             slice_idx=0,
         )
 
-        assert overlaps_measure.valid_overlaps.tolist() == [
+        assert overlaps_measure.valid_bases.tolist() == [
             [
-                [1.0, 1.0, 1.0],
-                [1.0, 3.0, 3.0],
-                [1.0, 3.0, 3.0],
+                [3.0, 3.0, 3.0],
+                [3.0, 5.0, 5.0],
+                [3.0, 5.0, 5.0],
             ],
             [
-                [2.0, 2.0, 2.0],
-                [2.0, 2.0, 2.0],
-                [2.0, 2.0, 2.0],
+                [3.0, 3.0, 3.0],
+                [3.0, 5.0, 5.0],
+                [3.0, 5.0, 5.0],
             ],
         ]
 
@@ -1565,53 +1556,22 @@ class DescribeCubemeasure(object):
             slice_idx=0,
         )
 
-        assert overlaps_measure.overlaps.tolist() == [
-            #
-            # --- Missing categories are not shown
-            #
-            # [
-            #     # CA_CATS: No Data
-            #     # ----------------
-            #     # A, B, C
-            #     [2, 1, 1],  # A
-            #     [1, 1, 1],  # B
-            #     [1, 1, 1],  # C
-            # ],
+        assert overlaps_measure.selected_bases.tolist() == [
             [
-                # CA_CATS: Poor
-                # -------------
-                # A, B, C
-                [1, 0, 0],  # A
-                [0, 0, 0],  # B
-                [0, 0, 0],  # C
+                [1.0, 0.0, 0.0],
+                [0.0, 1.0, 1.0],
+                [0.0, 1.0, 2.0],
             ],
             [
-                # CA_CATS: Fair
-                # -------------
-                # A, B, C
-                [0, 0, 0],  # A
-                [0, 1, 1],  # B
-                [0, 1, 2],  # C
+                [1.0, 0.0, 0.0],
+                [0.0, 1.0, 1.0],
+                [0.0, 1.0, 2.0],
             ],
             [
-                # CA_CATS: Good
-                # -------------
-                # A, B, C
-                [0, 0, 0],  # A
-                [0, 0, 0],  # B
-                [0, 0, 0],  # C
+                [1.0, 0.0, 0.0],
+                [0.0, 1.0, 1.0],
+                [0.0, 1.0, 2.0],
             ],
-            #
-            # --- Missing categories are not shown
-            #
-            # [
-            #     # CA_CATS: Not Shown
-            #     # ------------------
-            #     # A, B, C
-            #     [0, 0, 0],  # A
-            #     [0, 0, 0],  # B
-            #     [0, 0, 0],  # C
-            # ],
         ]
 
     def it_provides_valid_overlaps_for_ca_sub_x_ca_cat_x_mr_sub_x_mr_sel_subvar_0(self):
@@ -1623,21 +1583,21 @@ class DescribeCubemeasure(object):
             slice_idx=0,
         )
 
-        assert overlaps_measure.valid_overlaps.tolist() == [
+        assert overlaps_measure.valid_bases.tolist() == [
             [
-                [1.0, 1.0, 1.0],
-                [1.0, 1.0, 1.0],
-                [1.0, 1.0, 1.0],
+                [3.0, 3.0, 3.0],
+                [3.0, 4.0, 4.0],
+                [3.0, 4.0, 4.0],
             ],
             [
-                [2.0, 2.0, 2.0],
-                [2.0, 3.0, 3.0],
-                [2.0, 3.0, 3.0],
+                [3.0, 3.0, 3.0],
+                [3.0, 4.0, 4.0],
+                [3.0, 4.0, 4.0],
             ],
             [
-                [0.0, 0.0, 0.0],
-                [0.0, 0.0, 0.0],
-                [0.0, 0.0, 0.0],
+                [3.0, 3.0, 3.0],
+                [3.0, 4.0, 4.0],
+                [3.0, 4.0, 4.0],
             ],
         ]
 
@@ -1650,53 +1610,22 @@ class DescribeCubemeasure(object):
             slice_idx=1,
         )
 
-        assert overlaps_measure.overlaps.tolist() == [
-            #
-            # --- Missing categories are not shown
-            #
-            # [
-            #     # CA_CATS: Missing
-            #     # ----------------
-            #     # A, B, C
-            #     [1, 1, 1],  # A
-            #     [1, 1, 1],  # B
-            #     [1, 1, 1],  # C
-            # ],
+        assert overlaps_measure.selected_bases.tolist() == [
             [
-                # CA_CATS: Poor
-                # -------------
-                # A, B, C
-                [1, 0, 0],  # A
-                [0, 1, 1],  # B
-                [0, 1, 1],  # C
+                [2.0, 0.0, 0.0],
+                [0.0, 1.0, 1.0],
+                [0.0, 1.0, 2.0],
             ],
             [
-                # CA_CATS: Fair
-                # -------------
-                # A, B, C
-                [1, 0, 0],  # A
-                [0, 0, 0],  # B
-                [0, 0, 0],  # C
+                [2.0, 0.0, 0.0],
+                [0.0, 1.0, 1.0],
+                [0.0, 1.0, 2.0],
             ],
             [
-                # CA_CATS: Good
-                # -------------
-                # A, B, C
-                [0, 0, 0],  # A
-                [0, 0, 0],  # B
-                [0, 0, 1],  # C
+                [2.0, 0.0, 0.0],
+                [0.0, 1.0, 1.0],
+                [0.0, 1.0, 2.0],
             ],
-            #
-            # --- Missing categories are not shown
-            #
-            # [
-            #     # CA_CATS: Not Shown
-            #     # ------------------
-            #     # A, B, C
-            #     [0, 0, 0],  # A
-            #     [0, 0, 0],  # B
-            #     [0, 0, 0],  # C
-            # ],
         ]
 
     def it_provides_valid_overlaps_for_ca_sub_x_ca_cat_x_mr_sub_x_mr_sel_subvar_1(self):
@@ -1708,21 +1637,21 @@ class DescribeCubemeasure(object):
             slice_idx=1,
         )
 
-        assert overlaps_measure.valid_overlaps.tolist() == [
+        assert overlaps_measure.valid_bases.tolist() == [
             [
-                [2.0, 2.0, 2.0],
-                [2.0, 3.0, 3.0],
-                [2.0, 3.0, 3.0],
+                [4.0, 4.0, 4.0],
+                [4.0, 6.0, 6.0],
+                [4.0, 6.0, 6.0],
             ],
             [
-                [2.0, 2.0, 2.0],
-                [2.0, 2.0, 2.0],
-                [2.0, 2.0, 2.0],
+                [4.0, 4.0, 4.0],
+                [4.0, 6.0, 6.0],
+                [4.0, 6.0, 6.0],
             ],
             [
-                [0.0, 0.0, 0.0],
-                [0.0, 1.0, 1.0],
-                [0.0, 1.0, 1.0],
+                [4.0, 4.0, 4.0],
+                [4.0, 6.0, 6.0],
+                [4.0, 6.0, 6.0],
             ],
         ]
 
@@ -1783,3 +1712,115 @@ class Describe_BaseOrderHelper(object):
         )
 
         assert column_display_order.tolist() == expected_value
+
+
+class Describe_SortRowsByColumnValueHelper(object):
+    """Integration-test suite for `cr.cube.matrix._SortRowsByColumnValueHelper`."""
+
+    @pytest.mark.parametrize(
+        "fixture, measure, element_id, direction, expected_value",
+        (
+            (CR.MR_X_CAT, "col_base_unweighted", 4, "descending", [4, 3, 2, 1, 0]),
+            (CR.MR_X_CAT, "col_base_unweighted", 4, "ascending", [0, 1, 2, 3, 4]),
+            (CR.MR_X_CAT, "col_base_weighted", 1, "descending", [4, 0, 3, 1, 2]),
+            (CR.MR_X_CAT, "col_base_weighted", 1, "ascending", [2, 1, 3, 0, 4]),
+            (
+                CR.CAT_HS_X_MR_WGTD,
+                "col_percent",
+                1,
+                "descending",
+                [-4, -2, -3, -1, 1, 0, 2, 3, 4],
+            ),
+            (
+                CR.CAT_HS_X_MR_WGTD,
+                "col_percent",
+                1,
+                "ascending",
+                [4, 3, 2, 0, 1, -1, -3, -2, -4],
+            ),
+            (
+                CR.CAT_HS_X_CAT_HS,
+                "count_unweighted",
+                1,
+                "descending",
+                [-2, -1, 1, 4, 2, 3, 0],
+            ),
+            (
+                CR.CAT_HS_X_MR_WGTD,
+                "count_weighted",
+                1,
+                "ascending",
+                [4, 3, 2, 0, 1, -1, -3, -2, -4],
+            ),
+            (CR.CAT_X_CAT_MEAN_WGTD, "mean", 1, "descending", [3, 0, 2, 1, 4]),
+            (CR.MR_X_CAT, "row_base_unweighted", 1, "descending", [4, 3, 2, 1, 0]),
+            (CR.MR_X_CAT, "row_base_weighted", 2, "ascending", [0, 1, 2, 3, 4]),
+            (CR.CA_SUBVAR_X_CA_CAT_HS, "row_percent", 3, "descending", [1, 2, 0]),
+            (
+                CR.MR_X_CAT_2,
+                "table_base_unweighted",
+                1,
+                "ascending",
+                [3, 5, 4, 2, 1, 0],
+            ),
+            (CR.MR_X_CAT_2, "table_base_weighted", 1, "ascending", [3, 5, 4, 2, 1, 0]),
+            (CR.MR_X_CAT, "z_score", 2, "ascending", [3, 2, 1, 0, 4]),
+        ),
+    )
+    def it_can_compute_a_sort_by_column_values_row_order(
+        self, fixture, measure, element_id, direction, expected_value
+    ):
+        transforms = {
+            "rows_dimension": {
+                "order": {
+                    "type": "opposing_element",
+                    "element_id": element_id,
+                    "measure": measure,
+                    "direction": direction,
+                }
+            }
+        }
+        assembler = _Slice(Cube(fixture), 0, transforms, None, 0)._assembler
+
+        assert assembler._row_order.tolist() == expected_value
+
+
+class Describe_SortRowsByInsertedColumnHelper(object):
+    """Integration-test suite for `cr.cube.matrix._SortRowsByInsertedColumnHelper`."""
+
+    @pytest.mark.parametrize(
+        "fixture, measure, direction, fixed, expected_value",
+        (
+            (
+                CR.CAT_HS_X_CAT_HS,
+                "col_percent",
+                "descending",
+                {},
+                [-1, -2, 4, 3, 1, 2, 0],
+            ),
+            (
+                CR.CA_SUBVAR_X_CA_CAT_HS,
+                "col_percent",
+                "ascending",
+                {},
+                [2, 1, 0],
+            ),
+        ),
+    )
+    def it_can_compute_a_sort_by_insertion_values_row_order(
+        self, fixture, measure, direction, fixed, expected_value
+    ):
+        transforms = {
+            "rows_dimension": {
+                "order": {
+                    "type": "opposing_insertion",
+                    "insertion_id": 1,
+                    "measure": measure,
+                    "direction": direction,
+                    "fixed": fixed,
+                }
+            }
+        }
+        assembler = _Slice(Cube(fixture), 0, transforms, None, 0)._assembler
+
+        assert assembler._row_order.tolist() == expected_value
