@@ -324,3 +324,128 @@ class DescribeNumericArrays(object):
             ),
             nan_ok=True,
         )
+
+    def it_provides_unweighted_valid_counts_for_num_array_grouped_by_cat(self):
+        slice_ = Cube(NA.NUM_ARR_SUM_GROUPED_BY_CAT).partitions[0]
+
+        # unweighted valid counts
+        assert slice_.unweighted_counts.tolist() == [
+            [3.0, 2.0],
+            [3.0, 2.0],
+            [3.0, 2.0],
+        ]
+
+    def it_provides_valid_counts_for_num_array_grouped_by_cat_hs(self):
+        transforms = {
+            "columns_dimension": {
+                "insertions": [
+                    {
+                        # Valid counts will be nan
+                        # SubDiff doesn't be applied to valid counts
+                        "function": "subtotal",
+                        "name": "DIFF B-A",
+                        "args": [1, 3],
+                        "anchor": "top",
+                        "kwargs": {"negative": [1, 2]},
+                    },
+                    {
+                        "function": "subtotal",
+                        "anchor": 1,
+                        "args": [1, 2],
+                        "name": '"A" countries',
+                    },
+                    {
+                        # Valid counts will be nan
+                        # SubDiff doesn't be applied to valid counts
+                        "function": "subtotal",
+                        "name": "DIFF A-B",
+                        "args": [1, 2],
+                        "anchor": "bottom",
+                        "kwargs": {"negative": [1, 2]},
+                    },
+                ]
+            }
+        }
+        cube = Cube(NA.NUM_ARR_MEANS_GROUPED_BY_CAT_WEIGHTED, transforms=transforms)
+        slice_ = cube.partitions[0]
+
+        # Numeric array renders valid counts weighted and unweighted
+        assert slice_.unweighted_counts == pytest.approx(
+            np.array(
+                [
+                    [np.nan, 3, 5, 2, np.nan],
+                    [np.nan, 3, 4, 1, np.nan],
+                    [np.nan, 1, 2, 1, np.nan],
+                ]
+            ),
+            nan_ok=True,
+        )
+        assert slice_.counts == pytest.approx(
+            np.array(
+                [
+                    [np.nan, 0.8, 1.5, 0.7, np.nan],
+                    [np.nan, 0.8, 1.2, 0.4, np.nan],
+                    [np.nan, 0.2, 0.5, 0.3, np.nan],
+                ]
+            ),
+            nan_ok=True,
+        )
+
+    def it_provides_unweighted_valid_counts_for_num_array_x_mr(self):
+        slice_ = Cube(NA.NUM_ARR_MEANS_X_MR).partitions[0]
+
+        # unweighted valid counts
+        assert slice_.unweighted_counts.tolist() == [
+            [38.0, 14.0, 6.0, 18.0, 38.0],
+            [38.0, 14.0, 6.0, 18.0, 38.0],
+        ]
+
+    def it_provides_weighted_and_unweighted_valid_counbts_for_num_array_x_mr(self):
+        slice_ = Cube(NA.NUM_ARR_MEANS_GROUPED_BY_CAT_WEIGHTED).partitions[0]
+
+        # unweighted valid counts
+        assert slice_.unweighted_counts.tolist() == [
+            [3.0, 2.0],
+            [3.0, 1.0],
+            [1.0, 1.0],
+        ]
+        # weighted valid counts
+        assert slice_.counts.tolist() == [
+            [0.8, 0.7],
+            [0.8, 0.4],
+            [0.2, 0.3],
+        ]
+
+    def it_provides_unweighted_valid_counts_for_numeric_array_with_no_grouping(self):
+        strand = Cube(NA.NUM_ARR_MEANS_NO_GROUPING).partitions[0]
+
+        # unweighted valid counts
+        assert strand.unweighted_counts.tolist() == [6.0, 6.0]
+
+    def it_provides_weighted_and_unweighted_valid_counts_for_num_arr_no_grouping_wgtd(
+        self,
+    ):
+        strand = Cube(NA.NUM_ARR_MEANS_NO_GROUPING_WEIGHTED).partitions[0]
+
+        # unweighted valid counts
+        assert strand.unweighted_counts.tolist() == [19.0, 16.0, 17.0, 20.0]
+        # weighted valid counts
+        assert strand.counts.tolist() == [10.33, 11.22, 18.99, 14.55]
+
+    def it_provides_weighted_and_unweighted_valid_counts_for_num_arr_x_mr_wgtd(self):
+        slice_ = Cube(NA.NUM_ARR_MEANS_X_MR_WEIGHTED).partitions[0]
+
+        # unweighted valid counts
+        assert slice_.unweighted_counts.tolist() == [
+            [4.0, 7.0],
+            [4.0, 7.0],
+            [4.0, 6.0],
+            [2.0, 6.0],
+        ]
+        # weighted valid counts
+        assert slice_.counts.tolist() == [
+            [2.33, 7.009],
+            [4.123, 6.777],
+            [4.67, 6.4],
+            [1.898, 6.1],
+        ]
