@@ -134,23 +134,23 @@ class TestStandardizedResiduals(TestCase):
     """Test cr.cube implementation of column family pairwise comparisons"""
 
     def test_ca_subvar_hs_x_cat_hs_pairwise_t_tests(self):
-        slice_ = Cube(CR.CA_SUBVAR_HS_X_CAT_HS).partitions[0]
+        slice_ = Cube(CR.CA_SUBVAR_X_CA_CAT_HS).partitions[0]
         actual = slice_.pairwise_significance_tests[1]
 
         np.testing.assert_array_almost_equal(
-            actual.t_stats, load_python_expression("ca-subvar-hs-x-cat-hs-pw-tstats")
+            actual.t_stats, load_python_expression("ca-subvar-x-ca-cat-hs-pw-tstats")
         )
         np.testing.assert_array_almost_equal(
-            actual.p_vals, load_python_expression("ca-subvar-hs-x-cat-hs-pw-pvals")
+            actual.p_vals, load_python_expression("ca-subvar-x-ca-cat-hs-pw-pvals")
         )
         # check new api also
         np.testing.assert_array_almost_equal(
             slice_.pairwise_significance_t_stats(1),
-            load_python_expression("ca-subvar-hs-x-cat-hs-pw-tstats"),
+            load_python_expression("ca-subvar-x-ca-cat-hs-pw-tstats"),
         )
         np.testing.assert_array_almost_equal(
             slice_.pairwise_significance_p_vals(1),
-            load_python_expression("ca-subvar-hs-x-cat-hs-pw-pvals"),
+            load_python_expression("ca-subvar-x-ca-cat-hs-pw-pvals"),
         )
 
     def test_cat_nps_numval_x_cat_scale_means_pariwise_t_tests(self):
@@ -375,134 +375,290 @@ class TestOverlapsPairwiseSignificance(TestCase):
     def test_pairwise_significance_cat_x_mr_sub_x_mr_sel_0th_subvar(self):
         slice_ = Cube(OL.CAT_X_MR_SUB_X_MR_SEL).partitions[0]
 
-        np.testing.assert_almost_equal(
-            slice_.pairwise_significance_t_stats(0),
-            [
-                [0.0, -1.22474487, -2.44948974],
-                [0.0, 1.41421356, 1.41421356],
-            ],
+        assert slice_.column_percentages.tolist() == [
+            [0.0, 100.0, 100.0],
+            [100.0, 0.0, 0.0],
+        ]
+        assert slice_.pairwise_significance_t_stats(0) == pytest.approx(
+            np.array(
+                [
+                    [0.0, 3.11420549, 2.61911361],
+                    [0.0, -3.11420549, -2.61911361],
+                ]
+            )
         )
-        np.testing.assert_almost_equal(
-            slice_.pairwise_significance_p_vals(0),
-            [
-                [0.0, 0.43590578, 0.24675171],
-                [0.0, np.nan, np.nan],
-            ],
+        assert slice_.pairwise_significance_p_vals(0) == pytest.approx(
+            np.array(
+                [
+                    [0.0, 0.05270861, 0.07906174],
+                    [0.0, 0.05270861, 0.07906174],
+                ]
+            )
         )
 
     def test_pairwise_significance_cat_x_mr_sub_x_mr_sel_1st_subvar(self):
         slice_ = Cube(OL.CAT_X_MR_SUB_X_MR_SEL).partitions[0]
 
-        np.testing.assert_almost_equal(
-            slice_.pairwise_significance_t_stats(1),
-            [
-                [1.22474487, 0.0, -1.22474487],
-                [-1.41421356, 0.0, np.nan],
-            ],
+        assert slice_.column_percentages.tolist() == [
+            [0.0, 100.0, 100.0],
+            [100.0, 0.0, 0.0],
+        ]
+        assert slice_.pairwise_significance_t_stats(1) == pytest.approx(
+            np.array(
+                [
+                    [-3.11420549, 0.0, 0.0],
+                    [3.11420549, 0.0, 0.0],
+                ]
+            )
         )
-        np.testing.assert_almost_equal(
-            slice_.pairwise_significance_p_vals(1),
-            [
-                [0.43590578, 0.0, 0.43590578],
-                [np.nan, 0.0, np.nan],
-            ],
+        assert slice_.pairwise_significance_p_vals(1) == pytest.approx(
+            np.array(
+                [
+                    [0.05270861, 0.0, 1.0],
+                    [0.05270861, 0.0, 1.0],
+                ]
+            )
         )
 
     def test_pairwise_significance_cat_x_mr_sub_x_mr_sel_2nd_subvar(self):
         slice_ = Cube(OL.CAT_X_MR_SUB_X_MR_SEL).partitions[0]
 
-        np.testing.assert_almost_equal(
-            slice_.pairwise_significance_t_stats(2),
-            [
-                [2.44948974, 1.22474487, 0.0],
-                [-1.41421356, np.nan, 0.0],
-            ],
+        assert slice_.column_percentages.tolist() == [
+            [0.0, 100.0, 100.0],
+            [100.0, 0.0, 0.0],
+        ]
+        assert slice_.pairwise_significance_t_stats(2) == pytest.approx(
+            np.array(
+                [
+                    [-2.61911361, 0.0, 0.0],
+                    [2.61911361, 0.0, 0.0],
+                ],
+            ),
         )
-        np.testing.assert_almost_equal(
-            slice_.pairwise_significance_p_vals(2),
-            [
-                [0.24675171, 0.43590578, 0.0],
-                [np.nan, np.nan, 0.0],
-            ],
+        assert slice_.pairwise_significance_p_vals(2) == pytest.approx(
+            np.array(
+                [
+                    [0.07906174, 1.0, 0.0],
+                    [0.07906174, 1.0, 0.0],
+                ],
+            ),
         )
 
     def test_pairwise_significance_cat_x_mr_realistic_example(self):
         slice_ = Cube(OL.CAT_X_MR_REALISTIC_EXAMPLE).partitions[0]
 
-        np.testing.assert_almost_equal(
-            slice_.pairwise_significance_t_stats(4),
-            [
+        assert slice_.column_percentages == pytest.approx(
+            np.array(
                 [
-                    -3.9146295258075634,
-                    1.5105264449340376,
-                    3.2266563272193363,
-                    7.071067811865473,
-                    0.0,
-                    41.383226415679914,
-                ],
-                [
-                    -2.2756474745819952,
-                    2.909837329589324,
-                    3.4013451656911036,
-                    5.1050358012989845,
-                    0.0,
-                    63.718129288296005,
-                ],
-            ],
+                    [52.7687, 52.5926, 51.5504, 47.6852, 51.3889, np.nan],
+                    [47.2313, 47.4074, 48.4496, 52.3148, 48.6111, np.nan],
+                ]
+            ),
+            nan_ok=True,
         )
-        np.testing.assert_almost_equal(
-            slice_.pairwise_significance_p_vals(4),
-            [
+        assert slice_.pairwise_significance_t_stats(4) == pytest.approx(
+            np.array(
                 [
-                    0.0001337008782529292,
-                    0.1328815484386987,
-                    0.0015191171147603821,
-                    4.5152770411505116e-11,
-                    0.0,
-                    0.0,
+                    [1.00337549, 0.64382181, 0.0773666, -1.3677023, 0.0, np.nan],
+                    [-1.00337549, -0.64382181, -0.0773666, 1.3677023, 0.0, np.nan],
                 ],
+            ),
+            nan_ok=True,
+        )
+        assert slice_.pairwise_significance_p_vals(4) == pytest.approx(
+            np.array(
                 [
-                    0.02435314838688285,
-                    0.004195089619284165,
-                    0.0008696393198390773,
-                    1.0369509280128142e-06,
-                    0.0,
-                    0.0,
+                    [0.31647509, 0.52017481, 0.93838264, 0.17241219, 0.0, np.nan],
+                    [0.31647509, 0.52017481, 0.93838264, 0.17241219, 0.0, np.nan],
                 ],
-            ],
+            ),
+            nan_ok=True,
         )
 
     def test_pairwise_significance_mr_x_mr(self):
         slice_ = Cube(OL.MR_X_MR).partitions[0]
 
-        np.testing.assert_almost_equal(
-            slice_.pairwise_significance_t_stats(1),
-            [
-                [-1.11803399, 0.0, 1.82574186],
-                [1.47709789, 0.0, 3.46410162],
-                [np.nan, 0.0, np.nan],
-            ],
+        assert slice_.column_percentages == pytest.approx(
+            np.array(
+                [
+                    [100.0, 66.66667, np.nan],
+                    [66.66667, 100.0, np.nan],
+                    [0.0, 0.0, np.nan],
+                ]
+            ),
+            nan_ok=True,
         )
-        np.testing.assert_almost_equal(
-            slice_.pairwise_significance_p_vals(1),
-            [
-                [0.34501476, 0.0, 0.1653704],
-                [0.21370636, 0.0, 0.02572142],
-                [np.nan, 0.0, np.nan],
-            ],
+        assert slice_.pairwise_significance_t_stats(1) == pytest.approx(
+            np.array(
+                [
+                    [1.22474487, 0.0, np.nan],
+                    [-1.22474487, np.nan, np.nan],
+                    [np.nan, np.nan, np.nan],
+                ],
+            ),
+            nan_ok=True,
         )
+        assert slice_.pairwise_significance_p_vals(1) == pytest.approx(
+            np.array(
+                [
+                    [0.28786413, 1.0, np.nan],
+                    [0.27521973, np.nan, np.nan],
+                    [np.nan, np.nan, np.nan],
+                ],
+            ),
+            nan_ok=True,
+        )
+
+    def test_pairwise_cat_x_mr_gender_x_all_pets_owned_with_weighted_counts(self):
+        slice_ = Cube(OL.CAT_X_MR_GENDER_X_ALL_PETS_OWNED).partitions[0]
+
+        assert slice_.column_percentages.tolist() == pytest.approx(
+            np.array(
+                [
+                    [66.6667, 14.28571, 50.0],
+                    [33.33333, 85.714286, 50.0],
+                ]
+            )
+        )
+
+        # Assert for first column (subvariable)
+        assert slice_.pairwise_significance_t_stats(0).tolist() == pytest.approx(
+            np.array(
+                [
+                    [0.0, -2.6315597, -1.76353],
+                    [0.0, 2.6315597, 1.76353],
+                ]
+            ),
+        )
+        assert slice_.pairwise_significance_p_vals(0) == pytest.approx(
+            np.array(
+                [
+                    [0.0, 0.01410448, 0.0879948],
+                    [0.0, 0.01410448, 0.0879948],
+                ]
+            ),
+        )
+
+        # Assert for second column (subvariable)
+        assert slice_.pairwise_significance_t_stats(1).tolist() == pytest.approx(
+            np.array(
+                [
+                    [2.63156, 0.0, 8.10444],
+                    [-2.63156, 0.0, -8.10444],
+                ]
+            ),
+        )
+        assert slice_.pairwise_significance_p_vals(1) == pytest.approx(
+            np.array(
+                [
+                    [0.01410448, 0, 0.025067e-06],
+                    [0.01410448, 0, 0.025067e-06],
+                ]
+            ),
+        )
+
+        # Assert for third column (subvariable)
+        assert slice_.pairwise_significance_t_stats(2).tolist() == pytest.approx(
+            np.array(
+                [
+                    [1.763531, -8.104439, 0.0],
+                    [-1.763531, 8.104439, 0.0],
+                ]
+            ),
+        )
+        assert slice_.pairwise_significance_p_vals(2) == pytest.approx(
+            np.array(
+                [
+                    [0.0879948, 0.025067e-06, 0],
+                    [0.0879948, 0.025067e-06, 0],
+                ]
+            ),
+        )
+
+    def test_pairwise_significance_indices(self):
+        transforms = {"pairwise_indices": {"alpha": [0.05, 0.13]}}
+        slice_ = Cube(
+            OL.CAT_X_MR_GENDER_X_ALL_PETS_OWNED, transforms=transforms
+        ).partitions[0]
+
+        assert slice_.column_percentages.tolist() == pytest.approx(
+            np.array(
+                [
+                    [66.6667, 14.28571, 50.0],
+                    [33.33333, 85.714286, 50.0],
+                ]
+            )
+        )
+
+        assert slice_.pairwise_indices.tolist() == [
+            [(1,), (), (1,)],
+            [(), (0, 2), ()],
+        ]
+
+        assert slice_.pairwise_indices_alt.tolist() == [
+            [(1, 2), (), (1,)],
+            [(), (0, 2), (0,)],
+        ]
+
+    def test_alt_pairwise_indices_without_alt_alpha(self):
+        slice_ = Cube(OL.CAT_X_MR_GENDER_X_ALL_PETS_OWNED).partitions[0]
+        assert slice_.pairwise_indices_alt is None
+
+    def test_pairwise_significance_all_empty(self):
+        # ---Keep the alpha value this small to demo the error found in cr.server
+        transforms = {"pairwise_indices": {"alpha": [0.0000000001]}}
+        slice_ = Cube(
+            OL.CAT_X_MR_GENDER_X_ALL_PETS_OWNED, transforms=transforms
+        ).partitions[0]
+
+        assert slice_.column_percentages.tolist() == pytest.approx(
+            np.array(
+                [
+                    [66.6667, 14.28571, 50.0],
+                    [33.33333, 85.714286, 50.0],
+                ]
+            )
+        )
+
+        assert slice_.pairwise_indices.tolist() == [
+            [(), (), ()],
+            [(), (), ()],
+        ]
+
+    def test_pairwise_sig_for_realistic_example_mr_x_mr(self):
+        slice_ = Cube(OL.MR_X_MR_REALISTIC_EXAMPLE).partitions[0]
+        assert slice_.pairwise_significance_t_stats(3) == pytest.approx(
+            np.array(
+                [
+                    [13.40, 9.62, -0.49, 0.0, 2.27, -24.08],
+                    [-0.58, 19.94, -4.76, 0.0, -2.88, -13.48],
+                    [-0.53, 0.70, 22.76, 0.0, 10.32, -19.63],
+                    [-18.79, -13.57, -20.00, 0.0, -13.14, -29.10],
+                    [-5.45, -0.99, -2.83, 0.0, 21.75, -12.29],
+                    [-0.76, 0.17, -3.35, 0.0, -1.07, 31.68],
+                ]
+            ),
+            abs=10e-2,
+        )
+
+    def test_pairwise_sig_for_mr_x_mr_vs_mr_single_subvar_x_mr(self):
+        mr_x_mr_slice = Cube(OL.MR_X_MR).partitions[0]
+        t_stats_mr_x_mr = mr_x_mr_slice.pairwise_significance_t_stats(1)
+        mr_subvar_x_mr_slice = Cube(OL.MR_SINGLE_SUBVAR_X_MR).partitions[0]
+        t_stats_mr_subvar_x_mr = mr_subvar_x_mr_slice.pairwise_significance_t_stats(1)
+        # Assert same row stats are the same in both cases (MR x MR and MR_SEL x MR)
+        np.testing.assert_array_equal(t_stats_mr_x_mr[0], t_stats_mr_subvar_x_mr[0])
 
 
 class TestMeanDifferenceSignificance(object):
     def test_mean_diff_significance_for_numeric_array_grouped_by_cat(self):
         slice_ = Cube(NA.NUM_ARR_MULTI_NUMERIC_MEASURES_GROUPED_BY_CAT).partitions[0]
-
         assert slice_.pairwise_significance_means_t_stats(0) == pytest.approx(
             np.array(
                 [
-                    [0.0, 0.7482866, 4.61524529, 5.29463547],
-                    [0.0, 4.64371041, 7.14738681, 3.67511747],
-                    [0.0, -3.13387218, -6.35310465, -2.3932706],
+                    [-0.0, -0.7482866, -4.61524529, -5.29463547],
+                    [-0.0, -4.64371041, -7.14738681, -3.67511747],
+                    [-0.0, 3.13387218, 6.35310465, 2.3932706],
                 ]
             )
         )
@@ -516,16 +672,93 @@ class TestMeanDifferenceSignificance(object):
             )
         )
 
+    def test_mean_diff_significance_for_numeric_array_grouped_by_cat_hs(self):
+        transforms = {
+            "columns_dimension": {
+                "insertions": [
+                    {
+                        "function": "subtotal",
+                        "name": "DIFF B-A",
+                        "args": [4, 5],
+                        "anchor": "top",
+                        "kwargs": {"negative": [1, 2, 3]},
+                    },
+                    {
+                        "function": "subtotal",
+                        "args": [1, 2, 3],
+                        "anchor": 1,
+                        "name": '"A" countries',
+                    },
+                    {
+                        "function": "subtotal",
+                        "args": [4, 5],
+                        "anchor": 2,
+                        "name": '"B" countries',
+                    },
+                    {
+                        "function": "subtotal",
+                        "name": "DIFF A-B",
+                        "args": [1, 2, 3],
+                        "anchor": "bottom",
+                        "kwargs": {"negative": [4, 5]},
+                    },
+                ]
+            }
+        }
+        slice_ = Cube(
+            NA.NUM_ARR_MEANS_GROUPED_BY_CAT_HS, transforms=transforms
+        ).partitions[0]
+
+        # Column (0,3,5,8) are subtotals and the hypothesis testing for a subtotal
+        # computable ATM.
+        for col in slice_.inserted_column_idxs:
+            assert slice_.pairwise_significance_means_t_stats(
+                col
+            ).tolist() == pytest.approx(
+                np.full(slice_.means.shape, np.nan), nan_ok=True
+            )
+        np.testing.assert_almost_equal(
+            slice_.pairwise_significance_means_t_stats(7),
+            load_python_expression("num-arr-means-grouped-by-cat-hs-t-stats-col-7"),
+        )
+        np.testing.assert_almost_equal(
+            slice_.pairwise_significance_means_p_vals(7),
+            load_python_expression("num-arr-means-grouped-by-cat-hs-p-vals-col-7"),
+        )
+        assert slice_.pairwise_means_indices.tolist() == [
+            [None, (), (7,), None, (), None, (), (), None],
+            [None, (), (), None, (), None, (), (), None],
+            [None, (), (1, 4, 7), None, (), None, (1, 4, 7), (), None],
+            [None, (), (), None, (), None, (), (), None],
+        ]
+
+        # Test no subtotals
+        slice_ = Cube(NA.NUM_ARR_MEANS_GROUPED_BY_CAT_HS).partitions[0]
+        assert slice_.pairwise_means_indices.tolist() == [
+            [(), (4,), (), (), ()],
+            [(), (), (), (), ()],
+            [(), (0, 2, 4), (), (0, 2, 4), ()],
+            [(), (), (), (), ()],
+        ]
+        np.testing.assert_almost_equal(
+            slice_.pairwise_significance_means_t_stats(4),
+            load_python_expression("num-arr-means-grouped-by-cat-t-stats-col-4"),
+        )
+        np.testing.assert_almost_equal(
+            slice_.pairwise_significance_means_p_vals(4),
+            load_python_expression("num-arr-means-grouped-by-cat-p-vals-col-4"),
+        )
+
     def test_mean_diff_significance_for_numeric_array_x_mr(self):
         slice_ = Cube(NA.NUM_ARR_MULTI_NUMERIC_MEASURES_X_MR).partitions[0]
 
         assert slice_.pairwise_significance_means_t_stats(0) == pytest.approx(
             np.array(
                 [
-                    [0.0, -1.3056921, 1.1842639],
-                    [0.0, -1.7117695, 1.6297401],
+                    [-0.0, 1.3056921, -1.1842639],
+                    [-0.0, 1.7117695, -1.6297401],
                     [np.nan, np.nan, np.nan],
-                    [np.nan, 5.8872721, 4.4442801],
+                    [np.nan, -5.8872721, -4.4442801],
                 ]
             ),
             nan_ok=True,
@@ -549,14 +782,14 @@ class TestMeanDifferenceSignificance(object):
         ).partitions[0]
 
         assert slice_.pairwise_means_indices.tolist() == [
-            [(), (), (0, 1), (0, 1)],
-            [(), (0,), (0,), (0,)],
-            [(1, 2, 3), (2,), (), (2,)],
+            [(2, 3), (2, 3), (), ()],
+            [(1, 2, 3), (), (), ()],
+            [(), (0,), (0, 1, 3), (0,)],
         ]
         assert slice_.pairwise_means_indices_alt.tolist() == [
-            [(), (), (0, 1), (0, 1)],
-            [(), (0,), (0, 3), (0,)],
-            [(1, 2, 3), (2,), (), (2,)],
+            [(2, 3), (2, 3), (), ()],
+            [(1, 2, 3), (), (), (2,)],
+            [(), (0,), (0, 1, 3), (0,)],
         ]
 
     def test_mean_diff_significance_is_not_available(self):
@@ -598,8 +831,8 @@ class TestMeanDifferenceSignificance(object):
             np.array(
                 [
                     [np.nan, np.nan, np.nan, np.nan],
-                    [-14.1421356, 0.0, -3.1066887, np.nan],
-                    [-1.75662013, 0.0, np.nan, np.nan],
+                    [14.1421356, -0.0, 3.1066887, np.nan],
+                    [1.75662013, -0.0, np.nan, np.nan],
                 ]
             ),
             nan_ok=True,
@@ -621,11 +854,105 @@ class TestMeanDifferenceSignificance(object):
 
         assert slice_.pairwise_means_indices.tolist() == [
             [(), (), (), ()],
-            [(), (0, 2), (0,), ()],
+            [(1, 2), (), (1,), ()],
             [(), (), (), ()],
         ]
         assert slice_.pairwise_means_indices_alt.tolist() == [
-            [(3,), (), (3,), ()],
-            [(), (0, 2), (0,), ()],
+            [(), (), (), (0, 2)],
+            [(1, 2), (), (1,), ()],
             [(), (), (), ()],
         ]
+
+    def test_mean_diff_significance_indices_num_array_grouped_by_cat_hs_weighted(self):
+        insertions = [
+            {
+                "function": "subtotal",
+                "args": [1, 2, 3],
+                "anchor": "top",
+                "name": '"A" countries',
+            },
+            {
+                "function": "subtotal",
+                "args": [4, 5],
+                "anchor": "top",
+                "name": '"B" countries',
+            },
+            {
+                "function": "subtotal",
+                "name": "DIFF A-B",
+                "args": [1, 2, 3],
+                "anchor": "top",
+                "kwargs": {"negative": [4, 5]},
+            },
+            {
+                "function": "subtotal",
+                "name": "DIFF B-A",
+                "args": [4, 5],
+                "anchor": "top",
+                "kwargs": {"negative": [1, 2, 3]},
+            },
+        ]
+        transforms = {
+            "pairwise_indices": {"alpha": [0.05, 0.01]},
+            "columns_dimension": {"insertions": insertions},
+        }
+        slice_ = Cube(
+            NA.NUM_ARR_MEANS_GROUPED_BY_CAT_HS_WEIGHTED, transforms=transforms
+        ).partitions[0]
+        assert slice_.pairwise_means_indices.tolist() == [
+            [None, None, None, None, (), (), (), (), ()],
+            [None, None, None, None, (), (), (), (), ()],
+            [None, None, None, None, (), (), (), (6,), ()],
+        ]
+        assert slice_.pairwise_means_indices_alt.tolist() == [
+            [None, None, None, None, (), (), (), (), ()],
+            [None, None, None, None, (), (), (), (), ()],
+            [None, None, None, None, (), (), (), (4, 6), (4, 6)],
+        ]
+
+        # Test no subtotals
+        transforms = {"pairwise_indices": {"alpha": [0.05, 0.01]}}
+        slice_ = Cube(
+            NA.NUM_ARR_MEANS_GROUPED_BY_CAT_HS_WEIGHTED, transforms=transforms
+        ).partitions[0]
+        assert slice_.pairwise_means_indices.tolist() == [
+            [(), (), (), (), ()],
+            [(), (), (), (), ()],
+            [(), (), (), (2,), ()],
+        ]
+        assert slice_.pairwise_means_indices_alt.tolist() == [
+            [(), (), (), (), ()],
+            [(), (), (), (), ()],
+            [(), (), (), (0, 2), (0, 2)],
+        ]
+
+        # Test pruning
+        transforms = {
+            "columns_dimension": {
+                "prune": True,
+                "elements": {"1": {"hide": True}},
+                "insertions": insertions,
+            },
+            "pairwise_indices": {"alpha": [0.05, 0.01]},
+        }
+        slice_ = Cube(
+            NA.NUM_ARR_MEANS_GROUPED_BY_CAT_HS_WEIGHTED, transforms=transforms
+        ).partitions[0]
+
+        assert slice_.pairwise_means_indices.tolist() == [
+            [None, None, None, None, (), (), (), ()],
+            [None, None, None, None, (), (), (), ()],
+            [None, None, None, None, (), (), (5,), ()],
+        ]
+        assert slice_.pairwise_means_indices_alt.tolist() == [
+            [None, None, None, None, (), (), (), ()],
+            [None, None, None, None, (), (), (), ()],
+            [None, None, None, None, (), (), (4, 5), (4, 5)],
+        ]
+
+    def test_mean_indices_cat_x_mr_subvar(self):
+        slice_ = Cube(CR.CAT_X_MR_SUBVAR_MEANS).partitions[0]
+
+        assert slice_.pairwise_means_indices == pytest.approx(
+            np.array([[(), (), (), ()]])
+        )
