@@ -6,14 +6,15 @@ import numpy as np
 import pytest
 
 from cr.cube.cube import Cube
-from cr.cube.dimension import Dimension, _Element, _Subtotal, _Subtotals
-from cr.cube.enums import COLLATION_METHOD as CM, DIMENSION_TYPE as DT
+from cr.cube.dimension import Dimension, _Element, _OrderSpec, _Subtotal, _Subtotals
+from cr.cube.enums import COLLATION_METHOD as CM, DIMENSION_TYPE as DT, MEASURE as M
 from cr.cube.matrix.assembler import (
     Assembler,
     _BaseOrderHelper,
     _ColumnOrderHelper,
     _RowOrderHelper,
-    _SortRowsByColumnValueHelper,
+    _SortRowsByBaseColumnHelper,
+    _SortRowsByInsertedColumnHelper,
 )
 from cr.cube.matrix.cubemeasure import (
     BaseCubeResultMatrix,
@@ -21,19 +22,30 @@ from cr.cube.matrix.cubemeasure import (
 )
 from cr.cube.matrix.measure import (
     _BaseSecondOrderMeasure,
+    _ColumnComparableCounts,
     _ColumnProportions,
+    _ColumnShareSum,
     _ColumnUnweightedBases,
     _ColumnWeightedBases,
+    _Means,
+    _RowComparableCounts,
+    _RowProportions,
+    _RowShareSum,
     _RowUnweightedBases,
     _RowWeightedBases,
     SecondOrderMeasures,
+    _StdDev,
+    _Sums,
+    _TotalShareSum,
     _TableUnweightedBases,
     _TableWeightedBases,
     _UnweightedCounts,
     _WeightedCounts,
+    _Zscores,
 )
 
 from ...unitutil import (
+    ANY,
     class_mock,
     instance_mock,
     method_mock,
@@ -47,15 +59,25 @@ class DescribeAssembler(object):
     @pytest.mark.parametrize(
         "measure_prop_name, MeasureCls",
         (
+            ("column_comparable_counts", _ColumnComparableCounts),
             ("column_proportions", _ColumnProportions),
+            ("column_share_sum", _ColumnShareSum),
             ("column_unweighted_bases", _ColumnUnweightedBases),
             ("column_weighted_bases", _ColumnWeightedBases),
+            ("means", _Means),
+            ("row_comparable_counts", _RowComparableCounts),
+            ("row_proportions", _RowProportions),
+            ("row_share_sum", _RowShareSum),
             ("row_unweighted_bases", _RowUnweightedBases),
             ("row_weighted_bases", _RowWeightedBases),
+            ("stddev", _StdDev),
+            ("sums", _Sums),
             ("table_unweighted_bases", _TableUnweightedBases),
             ("table_weighted_bases", _TableWeightedBases),
+            ("total_share_sum", _TotalShareSum),
             ("weighted_counts", _WeightedCounts),
             ("unweighted_counts", _UnweightedCounts),
+            ("zscores", _Zscores),
         ),
     )
     def it_assembles_various_measures(
@@ -124,43 +146,40 @@ class DescribeAssembler(object):
 
     def it_provides_a_1D_columns_base_for_a_CAT_X_cube_result(
         self,
-        _rows_dimension_prop_,
-        dimension_,
-        _cube_result_matrix_prop_,
-        cube_result_matrix_,
+        _measures_prop_,
+        second_order_measures_,
+        _assemble_matrix_,
+        SumSubtotals_,
+        dimensions_,
         _column_subtotals_prop_,
         _column_order_prop_,
         _assemble_vector_,
     ):
-        _rows_dimension_prop_.return_value = dimension_
-        dimension_.dimension_type = DT.CAT
-        _cube_result_matrix_prop_.return_value = cube_result_matrix_
-        cube_result_matrix_.columns_base = [1, 2, 3]
+        dimensions_[0].dimension_type = DT.CAT
+        second_order_measures_.columns_base = [1, 2, 3]
         _column_subtotals_prop_.return_value = [3, 5]
         _column_order_prop_.return_value = [0, -2, 1, 2, -1]
         _assemble_vector_.return_value = np.array([1, 3, 2, 3, 5])
-        assembler = Assembler(None, None, None)
+        assembler = Assembler(None, dimensions_, None)
 
         columns_base = assembler.columns_base
-
-        _assemble_vector_.assert_called_once_with(
-            assembler, [1, 2, 3], [3, 5], [0, -2, 1, 2, -1], diffs_nan=True
-        )
+        assert _assemble_vector_.call_args_list[0][0][0] == assembler
+        assert _assemble_vector_.call_args_list[0][0][2] == [3, 5]
+        assert _assemble_vector_.call_args_list[0][0][3] == [0, -2, 1, 2, -1]
+        assert _assemble_vector_.call_args_list[0][0][3] == [0, -2, 1, 2, -1]
+        assert _assemble_vector_.call_args_list[0][1] == {"diffs_nan": True}
         assert columns_base.tolist() == [1, 3, 2, 3, 5]
 
     def but_it_provides_a_2D_columns_base_for_an_MR_X_cube_result(
         self,
-        _rows_dimension_prop_,
-        dimensions_,
-        _cube_result_matrix_prop_,
-        cube_result_matrix_,
-        SumSubtotals_,
+        _measures_prop_,
+        second_order_measures_,
         _assemble_matrix_,
+        SumSubtotals_,
+        dimensions_,
     ):
-        _rows_dimension_prop_.return_value = dimensions_[0]
+        second_order_measures_.columns_base = [[1, 2], [3, 4]]
         dimensions_[0].dimension_type = DT.MR_SUBVAR
-        cube_result_matrix_.columns_base = np.array([[1, 2], [3, 4]])
-        _cube_result_matrix_prop_.return_value = cube_result_matrix_
         SumSubtotals_.blocks.return_value = [[[1], [2]], [[3], [4]]]
         _assemble_matrix_.return_value = [[1, 2, 3], [4, 5, 6], [7, 8, 9]]
         assembler = Assembler(None, dimensions_, None)
@@ -168,9 +187,6 @@ class DescribeAssembler(object):
         columns_base = assembler.columns_base
 
         assert SumSubtotals_.blocks.call_count == 1
-        np.testing.assert_array_equal(
-            SumSubtotals_.blocks.call_args_list[0][0][0], [[1, 2], [3, 4]]
-        )
         assert SumSubtotals_.blocks.call_args_list[0][0][1] == dimensions_
         _assemble_matrix_.assert_called_once_with(assembler, [[[1], [2]], [[3], [4]]])
         assert columns_base == [[1, 2, 3], [4, 5, 6], [7, 8, 9]]
@@ -213,7 +229,7 @@ class DescribeAssembler(object):
         columns_margin = assembler.columns_margin
 
         _assemble_vector_.assert_called_once_with(
-            assembler, [1, 2, 3], [3, 5], [0, -2, 1, 2, -1]
+            assembler, [1, 2, 3], [3, 5], [0, -2, 1, 2, -1], diffs_nan=True
         )
         assert columns_margin.tolist() == [1, 3, 2, 3, 5]
 
@@ -236,7 +252,69 @@ class DescribeAssembler(object):
 
         columns_margin = assembler.columns_margin
 
-        SumSubtotals_.blocks.assert_called_once_with([[1, 2], [3, 4]], dimensions_)
+        SumSubtotals_.blocks.assert_called_once_with(
+            [[1, 2], [3, 4]], dimensions_, diff_cols_nan=True
+        )
+        _assemble_matrix_.assert_called_once_with(assembler, [[[1], [2]], [[3], [4]]])
+        assert columns_margin == [[1, 2, 3], [4, 5, 6], [7, 8, 9]]
+
+    def it_provides_a_1D_columns_margin_proportion_for_a_CAT_X_cube_result(
+        self,
+        _rows_dimension_prop_,
+        dimension_,
+        _cube_result_matrix_prop_,
+        cube_result_matrix_,
+        _column_subtotals_prop_,
+        _column_order_prop_,
+        _assemble_vector_,
+    ):
+        _rows_dimension_prop_.return_value = dimension_
+        dimension_.dimension_type = DT.CAT
+        _cube_result_matrix_prop_.return_value = cube_result_matrix_
+        cube_result_matrix_.table_margin = 6.0
+        cube_result_matrix_.columns_margin = np.array([1, 2, 3])
+        _column_subtotals_prop_.return_value = [3, 5]
+        _column_order_prop_.return_value = [0, -2, 1, 2, -1]
+        _assemble_vector_.return_value = [1, 3, 2, 3, 5]
+        assembler = Assembler(None, None, None)
+
+        columns_margin = assembler.columns_margin_proportion
+
+        _assemble_vector_.assert_called_once_with(
+            assembler, ANY, [3, 5], [0, -2, 1, 2, -1]
+        )
+        assert _assemble_vector_.call_args.args[1].tolist() == [
+            1 / 6.0,
+            2 / 6.0,
+            3 / 6.0,
+        ]
+        assert columns_margin == [1, 3, 2, 3, 5]
+
+    def but_it_provides_a_2D_columns_margin_proportion_for_an_MR_X_cube_result(
+        self,
+        _rows_dimension_prop_,
+        dimensions_,
+        _cube_result_matrix_prop_,
+        cube_result_matrix_,
+        SumSubtotals_,
+        _assemble_matrix_,
+    ):
+        _rows_dimension_prop_.return_value = dimensions_[0]
+        dimensions_[0].dimension_type = DT.MR_SUBVAR
+        cube_result_matrix_.columns_margin = [[1, 2], [3, 4]]
+        cube_result_matrix_.table_margin = np.array([4, 6])
+        _cube_result_matrix_prop_.return_value = cube_result_matrix_
+        SumSubtotals_.blocks.return_value = [[[1], [2]], [[3], [4]]]
+        _assemble_matrix_.return_value = [[1, 2, 3], [4, 5, 6], [7, 8, 9]]
+        assembler = Assembler(None, dimensions_, None)
+
+        columns_margin = assembler.columns_margin_proportion
+
+        SumSubtotals_.blocks.assert_called_once_with(ANY, dimensions_)
+        assert SumSubtotals_.blocks.call_args.args[0].tolist() == [
+            pytest.approx([1 / 4.0, 2 / 6.0]),
+            pytest.approx([3 / 4.0, 4 / 6.0]),
+        ]
         _assemble_matrix_.assert_called_once_with(assembler, [[[1], [2]], [[3], [4]]])
         assert columns_margin == [[1, 2, 3], [4, 5, 6], [7, 8, 9]]
 
@@ -391,7 +469,7 @@ class DescribeAssembler(object):
         rows_margin = assembler.rows_margin
 
         _assemble_vector_.assert_called_once_with(
-            assembler, [1, 2, 3], [3, 5], [0, -2, 1, 2, -1]
+            assembler, [1, 2, 3], [3, 5], [0, -2, 1, 2, -1], diffs_nan=True
         )
         assert rows_margin == [1, 3, 2, 3, 5]
 
@@ -414,7 +492,69 @@ class DescribeAssembler(object):
 
         rows_margin = assembler.rows_margin
 
-        SumSubtotals_.blocks.assert_called_once_with([[1, 2], [3, 4]], dimensions_)
+        SumSubtotals_.blocks.assert_called_once_with(
+            [[1, 2], [3, 4]], dimensions_, diff_rows_nan=True
+        )
+        _assemble_matrix_.assert_called_once_with(assembler, [[[1], [2]], [[3], [4]]])
+        assert rows_margin == [[1, 2, 3], [4, 5, 6], [7, 8, 9]]
+
+    def it_provides_a_1D_rows_margin_proportion_for_an_X_CAT_cube_result(
+        self,
+        _columns_dimension_prop_,
+        dimension_,
+        _cube_result_matrix_prop_,
+        cube_result_matrix_,
+        _row_subtotals_prop_,
+        _row_order_prop_,
+        _assemble_vector_,
+    ):
+        _columns_dimension_prop_.return_value = dimension_
+        dimension_.dimension_type = DT.CAT
+        _cube_result_matrix_prop_.return_value = cube_result_matrix_
+        cube_result_matrix_.table_margin = 6.0
+        cube_result_matrix_.rows_margin = np.array([1, 2, 3])
+        _row_subtotals_prop_.return_value = [3, 5]
+        _row_order_prop_.return_value = [0, -2, 1, 2, -1]
+        _assemble_vector_.return_value = [1, 3, 2, 3, 5]
+        assembler = Assembler(None, None, None)
+
+        rows_margin = assembler.rows_margin_proportion
+
+        _assemble_vector_.assert_called_once_with(
+            assembler, ANY, [3, 5], [0, -2, 1, 2, -1]
+        )
+        assert _assemble_vector_.call_args.args[1].tolist() == [
+            1 / 6.0,
+            2 / 6.0,
+            3 / 6.0,
+        ]
+        assert rows_margin == [1, 3, 2, 3, 5]
+
+    def but_it_provides_a_2D_rows_margin_proportion_for_an_X_MR_cube_result(
+        self,
+        _columns_dimension_prop_,
+        dimensions_,
+        _cube_result_matrix_prop_,
+        cube_result_matrix_,
+        SumSubtotals_,
+        _assemble_matrix_,
+    ):
+        _columns_dimension_prop_.return_value = dimensions_[1]
+        dimensions_[1].dimension_type = DT.MR_SUBVAR
+        cube_result_matrix_.rows_margin = [[1, 2], [3, 4]]
+        cube_result_matrix_.table_margin = np.array([3, 7])
+        _cube_result_matrix_prop_.return_value = cube_result_matrix_
+        SumSubtotals_.blocks.return_value = [[[1], [2]], [[3], [4]]]
+        _assemble_matrix_.return_value = [[1, 2, 3], [4, 5, 6], [7, 8, 9]]
+        assembler = Assembler(None, dimensions_, None)
+
+        rows_margin = assembler.rows_margin_proportion
+
+        SumSubtotals_.blocks.assert_called_once_with(ANY, dimensions_)
+        assert SumSubtotals_.blocks.call_args.args[0].tolist() == [
+            pytest.approx([1 / 3.0, 2 / 7.0]),
+            pytest.approx([3 / 3.0, 4 / 7.0]),
+        ]
         _assemble_matrix_.assert_called_once_with(assembler, [[[1], [2]], [[3], [4]]])
         assert rows_margin == [[1, 2, 3], [4, 5, 6], [7, 8, 9]]
 
@@ -616,78 +756,6 @@ class DescribeAssembler(object):
         _assemble_matrix_.assert_called_once_with(assembler, [[[1], [2]], [[3], [4]]])
         assert table_stderrs == [[1, 3, 2], [4, 6, 5]]
 
-    def it_computes_zscores_for_a_CAT_X_CAT_slice(
-        self,
-        request,
-        dimensions_,
-        _rows_dimension_prop_,
-        _columns_dimension_prop_,
-        _cube_result_matrix_prop_,
-        cube_result_matrix_,
-        ZscoreSubtotals_,
-        _assemble_matrix_,
-    ):
-        _rows_dimension_prop_.return_value = instance_mock(
-            request, Dimension, dimension_type=DT.CAT
-        )
-        _columns_dimension_prop_.return_value = instance_mock(
-            request, Dimension, dimension_type=DT.CAT
-        )
-        cube_result_matrix_.zscores = [[1, 2], [3, 4]]
-        _cube_result_matrix_prop_.return_value = cube_result_matrix_
-        ZscoreSubtotals_.blocks.return_value = [[[4], [3]], [[2], [1]]]
-        _assemble_matrix_.return_value = [[1, 2, 3], [4, 5, 6]]
-        assembler = Assembler(None, dimensions_, None)
-
-        zscores = assembler.zscores
-
-        ZscoreSubtotals_.blocks.assert_called_once_with(
-            [[1, 2], [3, 4]], dimensions_, cube_result_matrix_
-        )
-        _assemble_matrix_.assert_called_once_with(assembler, [[[4], [3]], [[2], [1]]])
-        assert zscores == [[1, 2, 3], [4, 5, 6]]
-
-    @pytest.mark.parametrize(
-        "rows_dim_type, cols_dim_type",
-        (
-            (DT.CAT, DT.MR),
-            (DT.MR, DT.CAT),
-            (DT.MR, DT.MR),
-        ),
-    )
-    def but_it_provides_zscores_with_NaN_subtotals_when_cube_has_an_MR_dimension(
-        self,
-        request,
-        dimensions_,
-        _rows_dimension_prop_,
-        rows_dim_type,
-        _columns_dimension_prop_,
-        cols_dim_type,
-        _cube_result_matrix_prop_,
-        cube_result_matrix_,
-        NanSubtotals_,
-        _assemble_matrix_,
-    ):
-        _rows_dimension_prop_.return_value = instance_mock(
-            request, Dimension, dimension_type=rows_dim_type
-        )
-        _columns_dimension_prop_.return_value = instance_mock(
-            request, Dimension, dimension_type=cols_dim_type
-        )
-        cube_result_matrix_.zscores = [[1, 2], [3, 4]]
-        _cube_result_matrix_prop_.return_value = cube_result_matrix_
-        NanSubtotals_.blocks.return_value = [[[1], [np.nan]], [[3], []]]
-        _assemble_matrix_.return_value = [[1, np.nan, 3], [4, np.nan, 6]]
-        assembler = Assembler(None, dimensions_, None)
-
-        zscores = assembler.zscores
-
-        NanSubtotals_.blocks.assert_called_once_with([[1, 2], [3, 4]], dimensions_)
-        _assemble_matrix_.assert_called_once_with(
-            assembler, [[[1], [np.nan]], [[3], []]]
-        )
-        assert zscores == [[1, np.nan, 3], [4, np.nan, 6]]
-
     # === implementation methods/properties ===
 
     @pytest.mark.parametrize(
@@ -935,10 +1003,6 @@ class DescribeAssembler(object):
     def TableStdErrSubtotals_(self, request):
         return class_mock(request, "cr.cube.matrix.assembler.TableStdErrSubtotals")
 
-    @pytest.fixture
-    def ZscoreSubtotals_(self, request):
-        return class_mock(request, "cr.cube.matrix.assembler.ZscoreSubtotals")
-
 
 class Describe_BaseOrderHelper(object):
     """Unit test suite for `cr.cube.matrix.assembler._BaseOrderHelper` object."""
@@ -965,7 +1029,7 @@ class Describe_BaseOrderHelper(object):
     @pytest.mark.parametrize(
         "collation_method, HelperCls",
         (
-            (CM.OPPOSING_ELEMENT, _SortRowsByColumnValueHelper),
+            (CM.OPPOSING_ELEMENT, _SortRowsByBaseColumnHelper),
             (CM.EXPLICIT_ORDER, _RowOrderHelper),
             (CM.PAYLOAD_ORDER, _RowOrderHelper),
         ),
@@ -973,7 +1037,9 @@ class Describe_BaseOrderHelper(object):
     def it_dispatches_to_the_right_row_order_helper(
         self, request, dimensions_, second_order_measures_, collation_method, HelperCls
     ):
-        dimensions_[0].collation_method = collation_method
+        dimensions_[0].order_spec = instance_mock(
+            request, _OrderSpec, collation_method=collation_method
+        )
         helper_ = instance_mock(
             request, HelperCls, _display_order=np.array([-1, 1, -2, 2])
         )
@@ -1039,6 +1105,21 @@ class Describe_BaseOrderHelper(object):
 
         assert order_helper._empty_row_idxs == expected_value
 
+    def it_retrieves_the_measure_object_to_help(self, request):
+        property_mock(
+            request,
+            _BaseOrderHelper,
+            "_order_spec",
+            return_value=instance_mock(request, _OrderSpec, measure=M.COLUMN_PERCENT),
+        )
+        column_proportions_ = instance_mock(request, _ColumnProportions)
+        second_order_measures_ = instance_mock(
+            request, SecondOrderMeasures, column_proportions=column_proportions_
+        )
+        order_helper = _BaseOrderHelper(None, second_order_measures_)
+
+        assert order_helper._measure is column_proportions_
+
     def it_provides_access_to_the_rows_dimension_to_help(self, dimension_):
         order_helper = _BaseOrderHelper((dimension_, None), None)
         assert order_helper._rows_dimension is dimension_
@@ -1074,7 +1155,9 @@ class Describe_ColumnOrderHelper(object):
         property_mock(
             request, _ColumnOrderHelper, "_columns_dimension", return_value=dimension_
         )
-        dimension_.collation_method = collation_method
+        dimension_.order_spec = instance_mock(
+            request, _OrderSpec, collation_method=collation_method
+        )
         CollatorCls_ = class_mock(
             request, "cr.cube.matrix.assembler.%s" % collator_class_name
         )
@@ -1138,7 +1221,9 @@ class Describe_RowOrderHelper(object):
         property_mock(
             request, _RowOrderHelper, "_rows_dimension", return_value=dimension_
         )
-        dimension_.collation_method = collation_method
+        dimension_.order_spec = instance_mock(
+            request, _OrderSpec, collation_method=collation_method
+        )
         CollatorCls_ = class_mock(
             request, "cr.cube.matrix.assembler.%s" % collator_class_name
         )
@@ -1153,6 +1238,16 @@ class Describe_RowOrderHelper(object):
         CollatorCls_.display_order.assert_called_once_with(dimension_, (2, 4, 6))
         assert order == (1, -2, 3, 5, -1)
 
+    def it_provides_access_to_the_order_spec_to_help(self, request, dimension_):
+        property_mock(
+            request, _RowOrderHelper, "_rows_dimension", return_value=dimension_
+        )
+        order_spec_ = instance_mock(request, _OrderSpec)
+        dimension_.order_spec = order_spec_
+        order_helper = _SortRowsByBaseColumnHelper(None, None)
+
+        assert order_helper._order_spec is order_spec_
+
     @pytest.mark.parametrize(
         "prune, empty_column_idxs, expected_value",
         (
@@ -1186,16 +1281,17 @@ class Describe_RowOrderHelper(object):
         return instance_mock(request, Dimension)
 
 
-class Describe_SortRowsByColumnValueHelper(object):
-    """Unit test suite for `cr.cube.matrix.assembler._SortRowsByColumnValueHelper`."""
+class Describe_SortRowsByBaseColumnHelper(object):
+    """Unit test suite for `cr.cube.matrix.assembler._SortRowsByBaseColumnHelper`."""
 
-    def it_extracts_the_sort_column_idx_from_the_order_spec_dict_to_help(
-        self, _rows_dimension_prop_, dimension_, _order_dict_prop_
+    def it_derives_the_sort_column_idx_from_the_order_spec_to_help(
+        self, _columns_dimension_prop_, dimension_, _order_spec_prop_, order_spec_
     ):
-        _rows_dimension_prop_.return_value = dimension_
+        _columns_dimension_prop_.return_value = dimension_
         dimension_.element_ids = (1, 2, 3, 4, 5)
-        _order_dict_prop_.return_value = {"element_id": 3}
-        order_helper = _SortRowsByColumnValueHelper(None, None)
+        _order_spec_prop_.return_value = order_spec_
+        order_spec_.element_id = 3
+        order_helper = _SortRowsByBaseColumnHelper(None, None)
 
         assert order_helper._column_idx == 2
 
@@ -1205,38 +1301,34 @@ class Describe_SortRowsByColumnValueHelper(object):
         _measure_prop_.return_value = measure_
         measure_.blocks = [[np.arange(20).reshape(4, 5), None], [None, None]]
         _column_idx_prop_.return_value = 2
-        order_helper = _SortRowsByColumnValueHelper(None, None)
+        order_helper = _SortRowsByBaseColumnHelper(None, None)
 
         assert order_helper._element_values.tolist() == [2, 7, 12, 17]
 
-    def it_retrieves_the_measure_object_to_help(self, request, _order_dict_prop_):
-        column_proportions_ = instance_mock(request, _ColumnProportions)
-        second_order_measures_ = instance_mock(
-            request, SecondOrderMeasures, column_proportions=column_proportions_
-        )
-        _order_dict_prop_.return_value = {"measure": "col_percent"}
-        order_helper = _SortRowsByColumnValueHelper(None, second_order_measures_)
-
-        assert order_helper._measure is column_proportions_
-
     def but_it_raises_when_an_unsupported_sort_by_value_measure_is_requested(
-        self, _order_dict_prop_
+        self, _order_spec_prop_, order_spec_
     ):
-        _order_dict_prop_.return_value = {"measure": "foobar"}
-        order_helper = _SortRowsByColumnValueHelper(None, None)
+        _order_spec_prop_.return_value = order_spec_
+        order_spec_.measure = M.TOTAL_SHARE_SUM
+        order_helper = _SortRowsByBaseColumnHelper(None, None)
 
         with pytest.raises(NotImplementedError) as e:
             order_helper._measure
 
-        assert str(e.value) == "sort-by-value for measure 'foobar' is not yet supported"
+        assert str(e.value) == (
+            "sort-by-value for measure 'MEASURE.TOTAL_SHARE_SUM' is not yet supported"
+        )
 
-    def it_computes_the_sorted_element_order_to_help(
-        self, request, _rows_dimension_prop_, dimension_
-    ):
-        _rows_dimension_prop_.return_value = dimension_
+    def it_computes_the_sorted_element_order_to_help(self, request, dimension_):
         property_mock(
             request,
-            _SortRowsByColumnValueHelper,
+            _SortRowsByBaseColumnHelper,
+            "_rows_dimension",
+            return_value=dimension_,
+        )
+        property_mock(
+            request,
+            _SortRowsByBaseColumnHelper,
             "_element_values",
             # --- return type is ndarray in real life, but assert_called_once_with()
             # --- won't match on those, so use list instead.
@@ -1244,18 +1336,18 @@ class Describe_SortRowsByColumnValueHelper(object):
         )
         property_mock(
             request,
-            _SortRowsByColumnValueHelper,
+            _SortRowsByBaseColumnHelper,
             "_subtotal_values",
             return_value=[15, 19],  # --- ndarray in real life ---
         )
         property_mock(
-            request, _SortRowsByColumnValueHelper, "_empty_row_idxs", return_value=()
+            request, _SortRowsByBaseColumnHelper, "_empty_row_idxs", return_value=()
         )
         SortByValueCollator_ = class_mock(
             request, "cr.cube.matrix.assembler.SortByValueCollator"
         )
         SortByValueCollator_.display_order.return_value = (-1, -2, 0, 2, 1)
-        order_helper = _SortRowsByColumnValueHelper(None, None)
+        order_helper = _SortRowsByBaseColumnHelper(None, None)
 
         order = order_helper._order
 
@@ -1264,22 +1356,13 @@ class Describe_SortRowsByColumnValueHelper(object):
         )
         assert order == (-1, -2, 0, 2, 1)
 
-    def it_provides_access_to_the_order_dict_to_help(
-        self, _rows_dimension_prop_, dimension_
-    ):
-        _rows_dimension_prop_.return_value = dimension_
-        dimension_.order_dict = {"order": "dict"}
-        order_helper = _SortRowsByColumnValueHelper(None, None)
-
-        assert order_helper._order_dict == {"order": "dict"}
-
     def it_extracts_the_subtotal_values_to_help(
         self, _measure_prop_, measure_, _column_idx_prop_
     ):
         _measure_prop_.return_value = measure_
         measure_.blocks = [[None, None], [np.arange(10, 101, 10).reshape(2, 5), None]]
         _column_idx_prop_.return_value = 2
-        order_helper = _SortRowsByColumnValueHelper(None, None)
+        order_helper = _SortRowsByBaseColumnHelper(None, None)
 
         assert order_helper._subtotal_values.tolist() == [30, 80]
 
@@ -1287,7 +1370,11 @@ class Describe_SortRowsByColumnValueHelper(object):
 
     @pytest.fixture
     def _column_idx_prop_(self, request):
-        return property_mock(request, _SortRowsByColumnValueHelper, "_column_idx")
+        return property_mock(request, _SortRowsByBaseColumnHelper, "_column_idx")
+
+    @pytest.fixture
+    def _columns_dimension_prop_(self, request):
+        return property_mock(request, _SortRowsByBaseColumnHelper, "_columns_dimension")
 
     @pytest.fixture
     def dimension_(self, request):
@@ -1299,12 +1386,116 @@ class Describe_SortRowsByColumnValueHelper(object):
 
     @pytest.fixture
     def _measure_prop_(self, request):
-        return property_mock(request, _SortRowsByColumnValueHelper, "_measure")
+        return property_mock(request, _SortRowsByBaseColumnHelper, "_measure")
 
     @pytest.fixture
-    def _order_dict_prop_(self, request):
-        return property_mock(request, _SortRowsByColumnValueHelper, "_order_dict")
+    def order_spec_(self, request):
+        return instance_mock(request, _OrderSpec)
 
     @pytest.fixture
-    def _rows_dimension_prop_(self, request):
-        return property_mock(request, _SortRowsByColumnValueHelper, "_rows_dimension")
+    def _order_spec_prop_(self, request):
+        return property_mock(request, _SortRowsByBaseColumnHelper, "_order_spec")
+
+
+class Describe_SortRowsByInsertedColumnHelper(object):
+    """Unit test suite for `cr.cube.matrix.assembler._SortRowsByInsertedColumnHelper`."""
+
+    def it_extracts_the_element_values_to_help(
+        self, _measure_prop_, measure_, _insertion_idx_prop_
+    ):
+        _measure_prop_.return_value = measure_
+        measure_.blocks = [[None, np.arange(12).reshape(4, 3)], [None, None]]
+        _insertion_idx_prop_.return_value = 1
+        order_helper = _SortRowsByInsertedColumnHelper(None, None)
+
+        assert order_helper._element_values.tolist() == [1, 4, 7, 10]
+
+    def it_derives_the_sort_insertion_idx_from_the_order_spec_to_help(
+        self, _columns_dimension_prop_, dimension_, _order_spec_prop_, order_spec_
+    ):
+        _columns_dimension_prop_.return_value = dimension_
+        dimension_.insertion_ids = (1, 2, 3)
+        _order_spec_prop_.return_value = order_spec_
+        order_spec_.insertion_id = 2
+        order_helper = _SortRowsByInsertedColumnHelper(None, None)
+
+        assert order_helper._insertion_idx == 1
+
+    def it_computes_the_sorted_element_order_to_help(self, request, dimension_):
+        property_mock(
+            request,
+            _SortRowsByInsertedColumnHelper,
+            "_rows_dimension",
+            return_value=dimension_,
+        )
+        property_mock(
+            request,
+            _SortRowsByInsertedColumnHelper,
+            "_element_values",
+            # --- return type is ndarray in real life, but assert_called_once_with()
+            # --- won't match on those, so use list instead.
+            return_value=[16, 3, 12],
+        )
+        property_mock(
+            request,
+            _SortRowsByInsertedColumnHelper,
+            "_subtotal_values",
+            return_value=[15, 19],  # --- ndarray in real life ---
+        )
+        property_mock(
+            request, _SortRowsByInsertedColumnHelper, "_empty_row_idxs", return_value=()
+        )
+        SortByValueCollator_ = class_mock(
+            request, "cr.cube.matrix.assembler.SortByValueCollator"
+        )
+        SortByValueCollator_.display_order.return_value = (-1, -2, 0, 2, 1)
+        order_helper = _SortRowsByInsertedColumnHelper(None, None)
+
+        order = order_helper._order
+
+        SortByValueCollator_.display_order.assert_called_once_with(
+            dimension_, [16, 3, 12], [15, 19], ()
+        )
+        assert order == (-1, -2, 0, 2, 1)
+
+    def it_extracts_the_subtotal_values_to_help(
+        self, _measure_prop_, measure_, _insertion_idx_prop_
+    ):
+        _measure_prop_.return_value = measure_
+        measure_.blocks = [[None, None], [None, np.arange(10, 61, 10).reshape(2, 3)]]
+        _insertion_idx_prop_.return_value = 1
+        order_helper = _SortRowsByInsertedColumnHelper(None, None)
+
+        assert order_helper._subtotal_values.tolist() == [20, 50]
+
+    # fixture components ---------------------------------------------
+
+    @pytest.fixture
+    def _columns_dimension_prop_(self, request):
+        return property_mock(
+            request, _SortRowsByInsertedColumnHelper, "_columns_dimension"
+        )
+
+    @pytest.fixture
+    def dimension_(self, request):
+        return instance_mock(request, Dimension)
+
+    @pytest.fixture
+    def _insertion_idx_prop_(self, request):
+        return property_mock(request, _SortRowsByInsertedColumnHelper, "_insertion_idx")
+
+    @pytest.fixture
+    def measure_(self, request):
+        return instance_mock(request, _BaseSecondOrderMeasure)
+
+    @pytest.fixture
+    def _measure_prop_(self, request):
+        return property_mock(request, _SortRowsByInsertedColumnHelper, "_measure")
+
+    @pytest.fixture
+    def order_spec_(self, request):
+        return instance_mock(request, _OrderSpec)
+
+    @pytest.fixture
+    def _order_spec_prop_(self, request):
+        return property_mock(request, _SortRowsByInsertedColumnHelper, "_order_spec")
