@@ -7,6 +7,7 @@ import pytest
 
 from cr.cube.cube import Cube
 from cr.cube.dimension import Dimension
+from cr.cube.enums import DIMENSION_TYPE as DT
 from cr.cube.matrix.cubemeasure import (
     CubeMeasures,
     _BaseUnweightedCubeCounts,
@@ -14,10 +15,12 @@ from cr.cube.matrix.cubemeasure import (
 )
 from cr.cube.matrix.measure import (
     _BaseSecondOrderMeasure,
+    _ColumnComparableCounts,
     _ColumnProportions,
     _ColumnShareSum,
     _ColumnUnweightedBases,
     _ColumnWeightedBases,
+    _RowComparableCounts,
     _RowProportions,
     _RowShareSum,
     _RowUnweightedBases,
@@ -29,6 +32,7 @@ from cr.cube.matrix.measure import (
     _TotalShareSum,
     _UnweightedCounts,
     _WeightedCounts,
+    _Zscores,
 )
 
 from ...unitutil import ANY, class_mock, instance_mock, property_mock
@@ -50,6 +54,7 @@ class DescribeSecondOrderMeasures(object):
             ("table_weighted_bases", _TableWeightedBases),
             ("weighted_counts", _WeightedCounts),
             ("unweighted_counts", _UnweightedCounts),
+            ("zscores", _Zscores),
         ),
     )
     def it_provides_access_to_various_measure_objects(
@@ -182,18 +187,54 @@ class Describe_BaseSecondOrderMeasure(object):
         return instance_mock(request, CubeMeasures)
 
 
+class Describe_ColumnComparableCounts(object):
+    """Unit test suite for `cr.cube.matrix.measure._ColumnComparableCounts` object."""
+
+    def it_computes_its_blocks(self, request, dimensions_, cube_measures_):
+        counts = np.arange(12).reshape(3, 4).tolist()
+        weighted_cube_counts_ = instance_mock(
+            request, _BaseWeightedCubeCounts, weighted_counts=counts
+        )
+        _weighted_cube_counts = property_mock(
+            request, _BaseSecondOrderMeasure, "_weighted_cube_counts"
+        )
+        _weighted_cube_counts.return_value = weighted_cube_counts_
+        SumSubtotals_ = class_mock(request, "cr.cube.matrix.measure.SumSubtotals")
+        SumSubtotals_.blocks.return_value = [[[1], [2]], [[3], [4]]]
+        col_comparable_counts = _ColumnComparableCounts(
+            dimensions_, None, cube_measures_
+        )
+
+        blocks = col_comparable_counts.blocks
+
+        SumSubtotals_.blocks.assert_called_once_with(counts, dimensions_, True)
+        assert blocks == [[[1], [2]], [[3], [4]]]
+
+    # fixture components ---------------------------------------------
+
+    @pytest.fixture
+    def cube_measures_(self, request):
+        return instance_mock(request, CubeMeasures)
+
+    @pytest.fixture
+    def dimensions_(self, request):
+        return instance_mock(request, Dimension), instance_mock(request, Dimension)
+
+
 class Describe_ColumnProportions(object):
     """Unit test suite for `cr.cube.matrix.measure._ColumnProportions` object."""
 
     def it_computes_its_blocks(self, request):
-        SumSubtotals_ = class_mock(request, "cr.cube.matrix.measure.SumSubtotals")
-        SumSubtotals_.blocks.return_value = [[5.0, 12.0], [21.0, 32.0]]
+        column_comparable_counts_ = instance_mock(
+            request, _ColumnComparableCounts, blocks=[[5.0, 12.0], [21.0, 32.0]]
+        )
         column_weighted_bases_ = instance_mock(
             request, _ColumnWeightedBases, blocks=[[5.0, 6.0], [7.0, 8.0]]
         )
         second_order_measures_ = instance_mock(
             request,
             SecondOrderMeasures,
+            column_comparable_counts=column_comparable_counts_,
             column_weighted_bases=column_weighted_bases_,
         )
         cube_measures_ = class_mock(request, "cr.cube.matrix.cubemeasure.CubeMeasures")
@@ -203,7 +244,6 @@ class Describe_ColumnProportions(object):
         )
 
         assert column_proportions.blocks == [[1.0, 2.0], [3.0, 4.0]]
-        SumSubtotals_.blocks.assert_called_once_with(ANY, None, diff_cols_nan=True)
 
 
 class Describe_ColumnUnweightedBases(object):
@@ -426,12 +466,47 @@ class Describe_ColumnWeightedBases(object):
         return property_mock(request, _ColumnWeightedBases, "_weighted_cube_counts")
 
 
+class Describe_RowComparableCounts(object):
+    """Unit test suite for `cr.cube.matrix.measure._RowComparableCounts` object."""
+
+    def it_computes_its_blocks(self, request, dimensions_, cube_measures_):
+        counts = np.arange(12).reshape(3, 4).tolist()
+        weighted_cube_counts_ = instance_mock(
+            request, _BaseWeightedCubeCounts, weighted_counts=counts
+        )
+        _weighted_cube_counts = property_mock(
+            request, _BaseSecondOrderMeasure, "_weighted_cube_counts"
+        )
+        _weighted_cube_counts.return_value = weighted_cube_counts_
+        SumSubtotals_ = class_mock(request, "cr.cube.matrix.measure.SumSubtotals")
+        SumSubtotals_.blocks.return_value = [[[1], [2]], [[3], [4]]]
+        row_comparable_counts = _RowComparableCounts(dimensions_, None, cube_measures_)
+
+        blocks = row_comparable_counts.blocks
+
+        SumSubtotals_.blocks.assert_called_once_with(
+            counts, dimensions_, diff_rows_nan=True
+        )
+        assert blocks == [[[1], [2]], [[3], [4]]]
+
+    # fixture components ---------------------------------------------
+
+    @pytest.fixture
+    def cube_measures_(self, request):
+        return instance_mock(request, CubeMeasures)
+
+    @pytest.fixture
+    def dimensions_(self, request):
+        return instance_mock(request, Dimension), instance_mock(request, Dimension)
+
+
 class Describe_RowProportions(object):
     """Unit test suite for `cr.cube.matrix.measure._RowProportions` object."""
 
     def it_computes_its_blocks(self, request):
-        SumSubtotals_ = class_mock(request, "cr.cube.matrix.measure.SumSubtotals")
-        SumSubtotals_.blocks.return_value = [[5.0, 12.0], [21.0, 32.0]]
+        row_comparable_counts_ = instance_mock(
+            request, _RowComparableCounts, blocks=[[5.0, 12.0], [21.0, 32.0]]
+        )
         row_weighted_bases_ = instance_mock(
             request, _RowWeightedBases, blocks=[[5.0, 6.0], [7.0, 8.0]]
         )
@@ -439,13 +514,13 @@ class Describe_RowProportions(object):
             request,
             SecondOrderMeasures,
             row_weighted_bases=row_weighted_bases_,
+            row_comparable_counts=row_comparable_counts_,
         )
         cube_measures_ = class_mock(request, "cr.cube.matrix.cubemeasure.CubeMeasures")
 
         row_proportions = _RowProportions(None, second_order_measures_, cube_measures_)
 
         assert row_proportions.blocks == [[1.0, 2.0], [3.0, 4.0]]
-        SumSubtotals_.blocks.assert_called_once_with(ANY, None, diff_rows_nan=True)
 
 
 class Describe_RowUnweightedBases(object):
@@ -1173,13 +1248,16 @@ class Describe_TableWeightedBases(object):
 class Describe_UnweightedCounts(object):
     """Unit test suite for `cr.cube.matrix.measure._UnweightedCounts` object."""
 
-    def it_computes_its_blocks_to_help(self, request, dimensions_):
+    def it_computes_its_blocks_to_help(self, request, dimensions_, cube_measures_):
         # --- these need to be in list form because the assert-called-with mechanism
         # --- uses equality, which doesn't work on numpy arrays. Normally this would be
         # --- the array itself.
         ucounts = np.arange(12).reshape(3, 4).tolist()
         unweighted_cube_counts_ = instance_mock(
-            request, _BaseUnweightedCubeCounts, unweighted_counts=ucounts
+            request,
+            _BaseUnweightedCubeCounts,
+            unweighted_counts=ucounts,
+            diff_nans=False,
         )
         property_mock(
             request,
@@ -1189,30 +1267,34 @@ class Describe_UnweightedCounts(object):
         )
         SumSubtotals_ = class_mock(request, "cr.cube.matrix.measure.SumSubtotals")
         SumSubtotals_.blocks.return_value = [[[1], [2]], [[3], [4]]]
-        unweighted_counts = _UnweightedCounts(dimensions_, None, None)
+        unweighted_counts = _UnweightedCounts(dimensions_, None, cube_measures_)
 
         blocks = unweighted_counts.blocks
 
-        SumSubtotals_.blocks.assert_called_once_with(ucounts, dimensions_)
+        SumSubtotals_.blocks.assert_called_once_with(ucounts, dimensions_, False, False)
         assert blocks == [[[1], [2]], [[3], [4]]]
 
     # fixture components ---------------------------------------------
 
     @pytest.fixture
     def dimensions_(self, request):
-        return (instance_mock(request, Dimension), instance_mock(request, Dimension))
+        return instance_mock(request, Dimension), instance_mock(request, Dimension)
+
+    @pytest.fixture
+    def cube_measures_(self, request):
+        return instance_mock(request, CubeMeasures)
 
 
 class Describe_WeightedCounts(object):
     """Unit test suite for `cr.cube.matrix.measure._WeightedCounts` object."""
 
-    def it_computes_its_blocks_to_help(self, request, dimensions_):
+    def it_computes_its_blocks_to_help(self, request, dimensions_, cube_measures_):
         # --- these need to be in list form because the assert-called-with mechanism
         # --- uses equality, which doesn't work on numpy arrays. Normally this would be
         # --- the array itself.
         counts = np.arange(12).reshape(3, 4).tolist()
         weighted_cube_counts_ = instance_mock(
-            request, _BaseWeightedCubeCounts, weighted_counts=counts
+            request, _BaseWeightedCubeCounts, weighted_counts=counts, diff_nans=False
         )
         property_mock(
             request,
@@ -1222,15 +1304,70 @@ class Describe_WeightedCounts(object):
         )
         SumSubtotals_ = class_mock(request, "cr.cube.matrix.measure.SumSubtotals")
         SumSubtotals_.blocks.return_value = [[[1], [2]], [[3], [4]]]
-        weighted_counts = _WeightedCounts(dimensions_, None, None)
+        weighted_counts = _WeightedCounts(dimensions_, None, cube_measures_)
 
         blocks = weighted_counts.blocks
 
-        SumSubtotals_.blocks.assert_called_once_with(counts, dimensions_)
+        SumSubtotals_.blocks.assert_called_once_with(counts, dimensions_, False, False)
         assert blocks == [[[1], [2]], [[3], [4]]]
 
     # fixture components ---------------------------------------------
 
+    @pytest.fixture
+    def cube_measures_(self, request):
+        return instance_mock(request, CubeMeasures)
+
     @pytest.fixture
     def dimensions_(self, request):
-        return (instance_mock(request, Dimension), instance_mock(request, Dimension))
+        return instance_mock(request, Dimension), instance_mock(request, Dimension)
+
+
+class Describe_Zscores(object):
+    """Unit test suite for `cr.cube.matrix.measure._Zscores` object."""
+
+    def it_computes_zscore_subtotals_blocks(self, request, dimensions_):
+        weighted_cube_counts_ = instance_mock(request, _BaseWeightedCubeCounts)
+        property_mock(
+            request,
+            _Zscores,
+            "_weighted_cube_counts",
+            return_value=weighted_cube_counts_,
+        )
+        ZscoreSubtotals_ = class_mock(request, "cr.cube.matrix.measure.ZscoreSubtotals")
+        ZscoreSubtotals_.blocks.return_value = [[[1], [2]], [[3], [4]]]
+        zscores = _Zscores(dimensions_, None, None)
+
+        blocks = zscores.blocks
+
+        ZscoreSubtotals_.blocks.assert_called_once_with(
+            weighted_cube_counts_, dimensions_
+        )
+        assert blocks == [[[1], [2]], [[3], [4]]]
+
+    def but_the_subtotal_blocks_are_NaNs_when_an_MR_dimension_is_present(
+        self, request, dimensions_
+    ):
+        weighted_cube_counts_ = instance_mock(
+            request, _BaseWeightedCubeCounts, zscores=[[1, 2], [3, 4]]
+        )
+        property_mock(
+            request,
+            _Zscores,
+            "_weighted_cube_counts",
+            return_value=weighted_cube_counts_,
+        )
+        NanSubtotals_ = class_mock(request, "cr.cube.matrix.measure.NanSubtotals")
+        NanSubtotals_.blocks.return_value = [[[1], [np.nan]], [[np.nan], [np.nan]]]
+        dimensions_[0].dimension_type = DT.MR_SUBVAR
+        zscores = _Zscores(dimensions_, None, None)
+
+        blocks = zscores.blocks
+
+        NanSubtotals_.blocks.assert_called_once_with([[1, 2], [3, 4]], dimensions_)
+        assert blocks == [[[1], [np.nan]], [[np.nan], [np.nan]]]
+
+    # fixture components ---------------------------------------------
+
+    @pytest.fixture
+    def dimensions_(self, request):
+        return instance_mock(request, Dimension), instance_mock(request, Dimension)
